#!/usr/bin/env python3
"""
Verify that a given JSON replay file fulfills basic expectations.
"""
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import argparse
import json
import jsonschema
import sys


expected_sensitive_value = \
    '''0000000 0000001 0000002 0000003 0000004 0000005 0000006 0000007 0000008 0000009 000000a 000000b 000000c 000000d ''' \
    '''000000e 000000f 0000010 0000011 0000012 0000013 0000014 0000015 0000016 0000017 0000018 0000019 000001a 000001b ''' \
    '''000001c 000001d 000001e 000001f 0000020 0000021 0000022 0000023 0000024 0000025 0000026 0000027 0000028 0000029 ''' \
    '''000002a 000002b 000002c 000002d 000002e 000002f 0000030 0000031 0000032 0000033 0000034 0000035 0000036 0000037 ''' \
    '''0000038 0000039 000003a 000003b 000003c 000003d 000003e 000003f 0000040 0000041 0000042 0000043 0000044 0000045 ''' \
    '''0000046 0000047 0000048 0000049 000004a 000004b 000004c 000004d 000004e 000004f 0000050 0000051 0000052 0000053 ''' \
    '''0000054 0000055 0000056 0000057 0000058 0000059 000005a 000005b 000005c 000005d 000005e 000005f 0000060 0000061 ''' \
    '''0000062 0000063 0000064 0000065 0000066 0000067 0000068 0000069 000006a 000006b 000006c 000006d 000006e 000006f ''' \
    '''0000070 0000071 0000072 0000073 0000074 0000075 0000076 0000077 0000078 0000079 000007a 000007b 000007c 000007d ''' \
    '''000007e 000007f 0000080 0000081 0000082 0000083 0000084 0000085 0000086 0000087 0000088 0000089 000008a 000008b ''' \
    '''000008c 000008d 000008e 000008f 0000090 0000091 0000092 0000093 0000094 0000095 0000096 0000097 0000098 0000099 ''' \
    '''000009a 000009b 000009c 000009d 000009e 000009f 00000a0 00000a1 00000a2 00000a3 00000a4 00000a5 00000a6 00000a7 ''' \
    '''00000a8 00000a9 00000aa 00000ab 00000ac 00000ad 00000ae 00000af 00000b0 00000b1 00000b2 00000b3 00000b4 00000b5 ''' \
    '''00000b6 00000b7 00000b8 00000b9 00000ba 00000bb 00000bc 00000bd 00000be 00000bf 00000c0 00000c1 00000c2 00000c3 ''' \
    '''00000c4 00000c5 00000c6 00000c7 00000c8 00000c9 00000ca 00000cb 00000cc 00000cd 00000ce 00000cf 00000d0 00000d1 ''' \
    '''00000d2 00000d3 00000d4 00000d5 00000d6 00000d7 00000d8 00000d9 00000da 00000db 00000dc 00000dd 00000de 00000df ''' \
    '''00000e0 00000e1 00000e2 00000e3 00000e4 00000e5 00000e6 00000e7 00000e8 00000e9 00000ea 00000eb 00000ec 00000ed ''' \
    '''00000ee 00000ef 00000f0 00000f1 00000f2 00000f3 00000f4 00000f5 00000f6 00000f7 00000f8 00000f9 00000fa 00000fb ''' \
    '''00000fc 00000fd 00000fe 00000ff'''


def validate_json(schema_json, replay_json):
    """
    Validate the replay file against the provided schema.
    """
    try:
        jsonschema.validate(instance=replay_json, schema=schema_json)
    except jsonschema.ValidationError as e:
        print("The replay file does not validate against the schema: {}".format(e))
        return False
    else:
        return True


def verify_there_was_a_transaction(replay_json):
    """
    Verify that the replay file has a sensible looking transaction.
    """
    try:
        transactions = replay_json['sessions'][0]['transactions']
    except KeyError:
        print("The replay file did not have transactions in it.")
        return False

    if len(transactions) < 1:
        print("There are no transactions in the replay file.")
        return False
    transaction = transactions[0]
    if not ('client-request' in transaction and 'proxy-response' in transaction):
        print("There was not request and response in the transaction of the replay file.")
        return False

    return True


def verify_http_information(replay_json):
    """
    Verify various expected HTTP information.
    """
    for session in replay_json['sessions']:
        top_layer = session['protocol'][0]
        is_http2 = top_layer['name'] == 'http' and top_layer['version'] == '2'
        if is_http2:
            # Every HTTP/2 client request should have a stream-id node.
            for transaction in session['transactions']:
                client_request = transaction['client-request']
                try:
                    client_request['http2']['stream-id']
                except KeyError:
                    print("An HTTP/2 transaction did not have a stream-id node.")
                    return False

    return True


def verify_request_target(replay_json, request_target):
    """
    Verify that the 'url' element of the first transaction contains the request target.
    """
    try:
        url = replay_json['sessions'][0]['transactions'][0]['client-request']['url']
    except KeyError:
        print("The replay file did not have a first transaction with a url element.")
        return False

    if url != request_target:
        print("Mismatched request target. Expected: {}, received: {}".format(request_target, url))
        return False
    return True


def verify_client_request_size(replay_json, client_request_size):
    """
    Verify that the 'url' element of the first transaction contains the request target.
    """
    try:
        size = int(replay_json['sessions'][0]['transactions'][0]['client-request']['content']['size'])
    except KeyError:
        print("The replay file did not have content size element in the first client-request.")
        return False

    if size != client_request_size:
        print("Mismatched client-request request size. Expected: {}, received: {}".format(client_request_size, size))
        return False
    return True


def verify_sensitive_fields_not_dumped(replay_json, sensitive_fields):
    """
    Verify that all of the cookie fields have the expected value.
    """
    message_types = ['client-request', 'proxy-request', 'server-response', 'proxy-response']
    try:
        for session in replay_json['sessions']:
            for transaction in session['transactions']:
                for message_type in transaction:
                    if message_type not in message_types:
                        continue
                    message = transaction[message_type]
                    for field in message['headers']['fields']:
                        field_name = field[0].lower()
                        if field_name in sensitive_fields:
                            field_value = field[1]
                            if field_value not in expected_sensitive_value:
                                print("Found an unexpected cookie: {}: {}".format(field[0], field[1]))
                                return False

    except KeyError:
        print("Could not find headers in the replay file.")
        return False

    return True


def verify_protocols_helper(replay_json, expected_protocols, is_client):
    try:
        if is_client:
            protocol_node = replay_json['sessions'][0]['protocol']
        else:
            protocol_node = replay_json['sessions'][0]['transactions'][0]['proxy-request']['protocol']

        dumped_protocols = ''
        is_first_protocol = True
        for protocol in protocol_node:
            if not is_first_protocol:
                dumped_protocols += ","
            is_first_protocol = False
            dumped_protocols += protocol['name']

        if dumped_protocols == expected_protocols:
            return True
        else:
            host_name = "client" if is_client else "server"
            print(
                'Unexpected protocol {} stack. Expected: "{}", found: "{}".'.format(
                    host_name, expected_protocols, dumped_protocols))
            return False
    except KeyError:
        print("Could not find {} protocol stack node in the replay file.".format(host_name))
        return False


def verify_client_protocols(replay_json, expected_protocols):
    return verify_protocols_helper(replay_json, expected_protocols, is_client=True)


def verify_server_protocols(replay_json, expected_protocols):
    return verify_protocols_helper(replay_json, expected_protocols, is_client=False)


def get_tls_features(replay_json, is_client):
    session = replay_json['sessions'][0]
    if is_client:
        protocol_parent = session['protocol']
    else:
        protocol_parent = session['transactions'][0]['proxy-request']['protocol']

    for protocol in protocol_parent:
        if protocol['name'] == "tls":
            return protocol
    else:
        raise KeyError('The "tls" protocol node was not found.')


def verify_tls_features(expected_tls_features, found_tls_features):
    for expected_tls_feature in expected_tls_features.split(','):
        expected_key, expected_value = expected_tls_feature.split(':')
        try:
            found_value = found_tls_features[expected_key]
        except KeyError:
            print("Could not find tls feature in the replay file: {}".format(expected_key))
            return False
        value_matches = False
        if isinstance(found_value, bool) or (found_value in ('true', 'false')):
            if expected_value.lower() == "false":
                expected_value = False
            elif expected_value.lower() == "true":
                expected_value = True
            else:
                raise ValueError("Cannot convert expected value to a boolean: {}, found: {}".format(expected_value, found_value))
            if found_value == 'true':
                found_value = True
            elif found_value == 'false':
                found_value = False
            value_matches = found_value == bool(expected_value)

        elif isinstance(found_value, str):
            value_matches = found_value == expected_value
        elif isinstance(found_value, int):
            value_matches = found_value == int(expected_value)
        else:
            raise ValueError("Cannot determine type of found value: {}".format(found_value))

        if not value_matches:
            print('Mismatched value for "{}", expected "{}", received "{}"'.format(expected_key, expected_value, found_value))
            return False
    return True


def verify_client_tls_features(replay_json, expected_tls_features):
    try:
        tls_features = get_tls_features(replay_json, is_client=True)
    except KeyError:
        print("Could not find client tls node in the replay file.")
        return False

    if not verify_tls_features(expected_tls_features, tls_features):
        print('Failed to verify client tls features in "{}"'.format(expected_tls_features))
        return False
    return True


def verify_server_tls_features(replay_json, expected_tls_features):
    try:
        tls_features = get_tls_features(replay_json, is_client=False)
    except KeyError:
        print("Could not find server tls node in the replay file.")
        return False

    if not verify_tls_features(expected_tls_features, tls_features):
        print('Failed to verify server tls features in "{}"'.format(expected_tls_features))
        return False
    return True


def verify_client_http_version(replay_json, expected_client_http_version):
    try:
        found_version = replay_json['sessions'][0]['transactions'][0]['client-request']['version']
    except KeyError:
        print("Could not find client-request:version node in the replay file.")
        return False

    if expected_client_http_version != found_version:
        print('Expected client version of "{}", but found "{}"'.format(expected_client_http_version, found_version))
        return False
    return True


def verify_client_request_body_bytes(replay_json, expected_body_bytes):
    """
    Verify that the replay file has the specified client request body bytes in it.
    """
    try:
        received_body_bytes = replay_json['sessions'][0]['transactions'][0]['client-request']['content']['data']
    except KeyError:
        print("The replay file did not have a client-request content data element in the first transaction.")
        return False

    if received_body_bytes != expected_body_bytes:
        print("Expected body bytes of '{0}' but got '{1}'".format(expected_body_bytes, received_body_bytes))
        return False

    # If the client request had that many bytes, verify that the proxy-request
    # does too. This is not guaranteed to always be true, but for our autest it
    # currently holds and this check is worthwhile.
    try:
        proxy_request_body_size = replay_json['sessions'][0]['transactions'][0]['proxy-request']['content']['size']
    except KeyError:
        print("The replay file did not have a proxy-request content size element in the first transaction.")
        return False

    if int(proxy_request_body_size) != len(expected_body_bytes):
        print(
            "Expected the proxy-request content size to be '{0}' but got '{1}'".format(
                len(expected_body_bytes), proxy_request_body_size))
        return False

    return True


def verify_proxy_response_body_bytes(replay_json, expected_body_bytes):
    """
    Verify that the replay file has the specified proxy response body bytes in it.
    """
    try:
        received_body_bytes = replay_json['sessions'][0]['transactions'][0]['proxy-response']['content']['data']
    except KeyError:
        print("The replay file did not have a proxy-response content data element in the first transaction.")
        return False

    if received_body_bytes != expected_body_bytes:
        print("Expected body bytes of '{0}' but got '{1}'".format(expected_body_bytes, received_body_bytes))
        return False

    # If the proxy response had that many bytes, verify that the server-response
    # does too. This is not guaranteed to always be true, but for our autest it
    # currently holds and this check is worthwhile.
    try:
        server_response_body_size = replay_json['sessions'][0]['transactions'][0]['server-response']['content']['size']
    except KeyError:
        print("The replay file did not have a server-response content size element in the first transaction.")
        return False

    if int(server_response_body_size) != len(expected_body_bytes):
        print(
            "Expected the server-response content size to be '{0}' but got '{1}'".format(
                len(expected_body_bytes), server_response_body_size))
        return False

    return True


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("schema_file", type=argparse.FileType('r'), help="The schema in which to validate the replay file.")
    parser.add_argument("replay_file", type=argparse.FileType('r'), help="The replay file to validate.")
    parser.add_argument("--request-target", help="The request target ('url' element) to expect in the replay file.")
    parser.add_argument("--client-request-size", type=int, help="The expected size value in the client-request node.")
    parser.add_argument(
        "--sensitive-fields",
        action="append",
        help="The fields that are considered sensitive and replaced with insensitive values.")
    parser.add_argument("--client-protocols", help="The comma-separated sequence of protocols to expect for the client connection.")
    parser.add_argument("--server-protocols", help="The comma-separated sequence of protocols to expect for the server connection.")
    parser.add_argument("--client-tls-features", help="The TLS values to expect for the client connection.")
    parser.add_argument("--server-tls-features", help="The TLS values to expect for the server connection.")
    parser.add_argument("--client-http-version", help="The client HTTP version to expect")
    parser.add_argument("--request_body", type=str, help="Verify that the client request has the specified body bytes.")
    parser.add_argument("--response_body", type=str, help="Verify that the proxy response has the specified body bytes.")
    return parser.parse_args()


def main():
    args = parse_args()

    schema_json = json.load(args.schema_file)
    replay_json = json.load(args.replay_file)

    if not validate_json(schema_json, replay_json):
        return 1

    # Verifying that there is a transaction in the replay file may seem
    # unnecessary since the replay file validated against the schema. But a JSON
    # file that doesn't have conflicting entry names will pass the schema. For
    # instance, this passes against our replay schema:
    #
    # {"name": "Bob", "languages": ["English", "French"]}
    #
    # Thus we do the following sanity check to make sure that the replay file
    # appears to have some transaction in it.
    if not verify_there_was_a_transaction(replay_json):
        return 1

    if not verify_http_information(replay_json):
        return 1

    if args.request_target and not verify_request_target(replay_json, args.request_target):
        return 1

    if args.client_request_size and not verify_client_request_size(replay_json, args.client_request_size):
        return 1

    if args.sensitive_fields and not verify_sensitive_fields_not_dumped(replay_json, args.sensitive_fields):
        return 1

    if args.client_protocols and not verify_client_protocols(replay_json, args.client_protocols):
        return 1

    if args.server_protocols and not verify_server_protocols(replay_json, args.server_protocols):
        return 1

    if args.client_tls_features and not verify_client_tls_features(replay_json, args.client_tls_features):
        return 1

    if args.server_tls_features and not verify_server_tls_features(replay_json, args.server_tls_features):
        return 1

    if args.client_http_version and not verify_client_http_version(replay_json, args.client_http_version):
        return 1

    if args.request_body and not verify_client_request_body_bytes(replay_json, args.request_body):
        return 1

    if args.response_body and not verify_proxy_response_body_bytes(replay_json, args.response_body):
        return 1

    return 0


if __name__ == '__main__':
    sys.exit(main())



"""Verify ALPN to origin functionality."""

#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from typing import Optional

Test.Summary = __doc__


class TestAlpnFunctionality:
    """Define an object to test a set of ALPN functionality."""

    _replay_file: str = 'tls_client_alpn_configuration.replay.yaml'
    _server_counter: int = 0
    _ts_counter: int = 0
    _client_counter: int = 0

    def __init__(
            self,
            records_config_alpn: Optional[str] = None,
            conf_remap_alpn: Optional[str] = None,
            alpn_is_malformed: bool = False):
        """Declare the various test Processes.

        :param records_config_alpn: The string with which to configure the ATS
        ALPN via proxy.config.ssl.client.alpn_protocols in the records.yaml.
        If the paramenter is None, then no ALPN configuration will be
        explicitly set and ATS will use the default value.

        :param conf_remap_alpn: The string with which to configure the Traffic
        Server ALPN proxy.config.http.alpn_protocols configuration via
        conf_remap. If the parameter is None, then no conf_remap configuration
        will be set.

        :param alpn_is_malformed: If True, then the configured ALPN string in
        the records.yaml will be malformed. The TestRun will be configured to
        expect a warning and the server will be configured to receive no ALPN.
        """
        self._alpn = records_config_alpn
        self._alpn_conf_remap_alpn = conf_remap_alpn
        self._alpn_is_malformed = alpn_is_malformed

        configured_alpn = records_config_alpn if conf_remap_alpn is None else conf_remap_alpn
        if alpn_is_malformed:
            configured_alpn = None
        self._server = self._configure_server(configured_alpn)

        self._ts = self._configure_trafficserver(records_config_alpn, conf_remap_alpn, alpn_is_malformed)

    def _configure_server(self, expected_alpn: Optional[str] = None):
        """Configure the test server.

        :param expected_alpn: The ALPN expected from the client. If this is
        None, then the server will not expect an ALPN value.
        """
        server = Test.MakeVerifierServerProcess(f'server-{TestAlpnFunctionality._server_counter}', self._replay_file)
        TestAlpnFunctionality._server_counter += 1

        if expected_alpn is None:
            server.Streams.stdout = Testers.ContainsExpression('Negotiated ALPN: none', 'Verify that ATS sent no ALPN string.')
        else:
            protocols = expected_alpn.split(',')
            for protocol in protocols:
                server.Streams.stdout = Testers.ContainsExpression(
                    f'ALPN.*:.*{protocol}', 'Verify that the server parsed the configured ALPN string from ATS.')
        return server

    def _configure_trafficserver(
            self,
            records_config_alpn: Optional[str] = None,
            conf_remap_alpn: Optional[str] = None,
            alpn_is_malformed: bool = False):
        """Configure a Traffic Server process.

        :param records_config_alpn: See the description of this parameter in
        TestAlpnFunctionality._init__.
        """
        ts = Test.MakeATSProcess(f'ts-{TestAlpnFunctionality._ts_counter}', enable_tls=True, enable_cache=False)
        TestAlpnFunctionality._ts_counter += 1

        ts.addDefaultSSLFiles()
        ts.Disk.records_config.update(
            {
                "proxy.config.ssl.server.cert.path": f'{ts.Variables.SSLDir}',
                "proxy.config.ssl.server.private_key.path": f'{ts.Variables.SSLDir}',
                "proxy.config.ssl.client.verify.server.policy": 'PERMISSIVE',
                'proxy.config.diags.debug.enabled': 3,
                'proxy.config.diags.debug.tags': 'ssl|http',
            })

        if records_config_alpn is not None:
            ts.Disk.records_config.update({
                'proxy.config.ssl.client.alpn_protocols': records_config_alpn,
            })

        ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

        conf_remap_specification = ''
        if conf_remap_alpn is not None:
            conf_remap_specification = (
                '@plugin=conf_remap.so '
                f'@pparam=proxy.config.ssl.client.alpn_protocols={conf_remap_alpn}')

        ts.Disk.remap_config.AddLine(f'map / https://127.0.0.1:{self._server.Variables.https_port} {conf_remap_specification}')

        if alpn_is_malformed:
            ts.Disk.diags_log.Content = Testers.ContainsExpression("ERROR.*ALPN", "There should be no ALPN parse warnings.")
        else:
            ts.Disk.diags_log.Content += Testers.ExcludesExpression("ERROR.*ALPN", "There should be no ALPN parse warnings.")

        return ts

    def run(self):
        """Configure the TestRun."""
        description = "default" if self._alpn is None else self._alpn
        tr = Test.AddTestRun(f'ATS ALPN configuration: {description}')
        tr.Processes.Default.StartBefore(self._server)
        tr.Processes.Default.StartBefore(self._ts)

        tr.AddVerifierClientProcess(
            f'client-{TestAlpnFunctionality._client_counter}', self._replay_file, https_ports=[self._ts.Variables.ssl_port])
        TestAlpnFunctionality._client_counter += 1


#
# Test default configuration.
#
TestAlpnFunctionality().run()

#
# Test various valid ALPN configurations.
#
TestAlpnFunctionality(records_config_alpn='http/1.1').run()
TestAlpnFunctionality(records_config_alpn='http/1.1,http/1.0').run()
TestAlpnFunctionality(records_config_alpn='http/1.1', conf_remap_alpn='http/1.1,http/1.0').run()
TestAlpnFunctionality(records_config_alpn='h2,http/1.1').run()
TestAlpnFunctionality(records_config_alpn='h2').run()

#
# Test malformed ALPN configurations.
#
TestAlpnFunctionality(records_config_alpn='not_a_protocol', alpn_is_malformed=True).run()
# Note that HTTP/3 to origin is not currently supported.
TestAlpnFunctionality(records_config_alpn='h3', alpn_is_malformed=True).run()



#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''
Test PUSHing an object into the cache and the GETting it with a few variations on the client connection protocol.
'''

# NOTE: You can also use this to test client-side communication when GET-ing very large (multi-GB) objects
# by increasing the value of the obj_kilobytes variable below.  (But do not increase it on any shared branch
# that we do CI runs on.)

Test.SkipUnless(Condition.HasCurlFeature('http2'))

ts = Test.MakeATSProcess("ts1", enable_tls=True)
ts.addDefaultSSLFiles()

ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http|dns|cache',
        'proxy.config.http.cache.required_headers': 0,  # No required headers for caching
        'proxy.config.http.push_method_enabled': 1,
        'proxy.config.proxy_name': 'Poxy_Proxy',  # This will be the server name.
        'proxy.config.ssl.server.cert.path': ts.Variables.SSLDir,
        'proxy.config.ssl.server.private_key.path': ts.Variables.SSLDir,
        'proxy.config.url_remap.remap_required': 0
    })

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

ts.Disk.remap_config.AddLine(f'map https://localhost:{ts.Variables.ssl_port} http://localhost:{ts.Variables.port}')
ts.Disk.remap_config.AddLine(f'map https://localhost:{ts.Variables.ssl_portv6} http://localhost:{ts.Variables.port}')

# Size of object to get.  (NOTE:  If you increase this significantly you may also have to increase cache
# capacity in tests/gold_tests/autest-size/min_cfg/storage.config.  Also, for very large objects, if
# proxy.config.diags.debug.enabled is 1, the PUSH request will timeout and fail.)
#
obj_kilobytes = 10 * 1024
obj_bytes = obj_kilobytes * 10
header = "HTTP/1.1 200 OK\r\nContent-length: {}\r\n\r\n".format(obj_bytes)


def create_pushfile():
    f = open(Test.RunDirectory + "/objfile", "w")
    f.write(header)
    f.write("x" * obj_bytes)
    f.close()
    return True


tr = Test.AddTestRun("PUSH an object to the cache")
# Delay on readiness of TS IPv4 ssl port
tr.Processes.Default.StartBefore(ts, ready=lambda: create_pushfile())
# Put object with URL http://localhost/bigobj in cache using PUSH request.
tr.Processes.Default.Command = "curl -v -H 'Content-Type: application/octet-stream' --data-binary @{}/objfile -X PUSH http://localhost:{}/bigobj -H 'Content-Length:{}'".format(
    Test.RunDirectory, ts.Variables.port,
    len(header) + obj_bytes)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression("HTTP/1.1 201 Created", "The PUSH request should have succeeded")

tr = Test.AddTestRun("GET bigobj: cleartext, HTTP/1.1, IPv4")
tr.Processes.Default.Command = f'curl --verbose --ipv4 --http1.1 http://localhost:{ts.Variables.port}/bigobj'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression("HTTP/1.1 200 OK", "Should fetch pushed object")
tr.Processes.Default.Streams.All = Testers.ContainsExpression("Content-length: 102400", "Content size should be accurate")

tr = Test.AddTestRun("GET bigobj: TLS, HTTP/1.1, IPv4")
tr.Processes.Default.Command = f'curl --verbose --ipv4 --http1.1 --insecure https://localhost:{ts.Variables.ssl_port}/bigobj'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression("HTTP/1.1 200 OK", "Should fetch pushed object")
tr.Processes.Default.Streams.All = Testers.ContainsExpression("Content-length: 102400", "Content size should be accurate")

tr = Test.AddTestRun("GET bigobj: TLS, HTTP/2, IPv4")
tr.Processes.Default.Command = f'curl --verbose --ipv4 --http2 --insecure https://localhost:{ts.Variables.ssl_port}/bigobj'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression("HTTP/2 200", "Should fetch pushed object")
tr.Processes.Default.Streams.All = Testers.ContainsExpression("content-length: 102400", "Content size should be accurate")

tr = Test.AddTestRun("GET bigobj: TLS, HTTP/2, IPv6")
tr.Processes.Default.Command = f'curl --verbose --ipv6 --http2 --insecure https://localhost:{ts.Variables.ssl_portv6}/bigobj'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression("HTTP/2 200", "Should fetch pushed object")
tr.Processes.Default.Streams.All = Testers.ContainsExpression("content-length: 102400", "Content size should be accurate")

# Verify that PUSH requests are rejected when push_method_enabled is 0 (the
# default configuration).
ts = Test.MakeATSProcess("ts2", enable_tls=True)
ts.addDefaultSSLFiles()

ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http|dns|cache',
        'proxy.config.http.cache.required_headers': 0,  # No required headers for caching
        'proxy.config.proxy_name': 'Poxy_Proxy',  # This will be the server name.
        'proxy.config.ssl.server.cert.path': ts.Variables.SSLDir,
        'proxy.config.ssl.server.private_key.path': ts.Variables.SSLDir,
        'proxy.config.url_remap.remap_required': 0
    })

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

ts.Disk.remap_config.AddLine(f'map https://localhost:{ts.Variables.ssl_port} http://localhost:{ts.Variables.port}')
ts.Disk.remap_config.AddLine(f'map https://localhost:{ts.Variables.ssl_portv6} http://localhost:{ts.Variables.port}')

tr = Test.AddTestRun("PUSH request is rejected when push_method_enabled is 0")
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.Command = "curl -v -H 'Content-Type: application/octet-stream' --data-binary @{}/objfile -X PUSH http://localhost:{}/bigobj -H 'Content-Length:{}'".format(
    Test.RunDirectory, ts.Variables.port,
    len(header) + obj_bytes)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression(
    "403 Access Denied", "The PUSH request should have received a 403 response.")



'''
Tests that HEAD requests return proper responses
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import sys

Test.Summary = '''
Tests that HEAD requests return proper responses
'''

ts = Test.MakeATSProcess("ts")

HOST = 'www.example.test'

server = Test.MakeOriginServer("server")

ts.Disk.remap_config.AddLine('map http://{0} http://127.0.0.1:{1}'.format(HOST, server.Variables.Port))

server.addResponse(
    "sessionfile.log", {
        "headers": "HEAD /head200 HTTP/1.1\r\nHost: {0}\r\n\r\n".format(HOST),
        "timestamp": "1469733493.993",
        "body": ""
    }, {
        "headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": "This body should not be returned for a HEAD request."
    })

server.addResponse(
    "sessionfile.log", {
        "headers": "GET /get200 HTTP/1.1\r\nHost: {0}\r\n\r\n".format(HOST),
        "timestamp": "1469733493.993",
        "body": ""
    }, {
        "headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": "This body should be returned for a GET request."
    })

server.addResponse(
    "sessionfile.log", {
        "headers": "GET /get304 HTTP/1.1\r\nHost: {0}\r\n\r\n".format(HOST),
        "timestamp": "1469733493.993",
        "body": ""
    }, {
        "headers": "HTTP/1.1 304 Not Modified\r\nConnection: close\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    })

Test.Setup.Copy(os.path.join(os.pardir, os.pardir, 'tools', 'tcp_client.py'))
Test.Setup.Copy('data')

trhead200 = Test.AddTestRun("Test domain {0}".format(HOST))
trhead200.Processes.Default.StartBefore(Test.Processes.ts)
trhead200.Processes.Default.StartBefore(server, ready=When.PortOpen(server.Variables.Port))
trhead200.StillRunningAfter = ts
trhead200.StillRunningAfter = server

trhead200.Processes.Default.Command = f"{sys.executable} tcp_client.py 127.0.0.1 {ts.Variables.port} data/{HOST}_head_200.txt"
trhead200.Processes.Default.TimeOut = 5  # seconds
trhead200.Processes.Default.ReturnCode = 0
trhead200.Processes.Default.Streams.stdout = "gold/http-head-200.gold"

trget200 = Test.AddTestRun("Test domain {0}".format(HOST))
trget200.StillRunningBefore = ts
trget200.StillRunningBefore = server
trget200.StillRunningAfter = ts
trget200.StillRunningAfter = server

trget200.Processes.Default.Command = f"{sys.executable} tcp_client.py 127.0.0.1 {ts.Variables.port} data/{HOST}_get_200.txt"
trget200.Processes.Default.TimeOut = 5  # seconds
trget200.Processes.Default.ReturnCode = 0
trget200.Processes.Default.Streams.stdout = "gold/http-get-200.gold"

trget304 = Test.AddTestRun("Test domain {0}".format(HOST))
trget304.StillRunningBefore = ts
trget304.StillRunningBefore = server
trget304.StillRunningAfter = ts
trget304.StillRunningAfter = server

cmd_tpl = f"{sys.executable} tcp_client.py 127.0.0.1 {ts.Variables.port} data/{HOST}_get_304.txt"
trget304.Processes.Default.Command = cmd_tpl
trget304.Processes.Default.TimeOut = 5  # seconds
trget304.Processes.Default.ReturnCode = 0
trget304.Processes.Default.Streams.stdout = "gold/http-get-304.gold"



"""Test proxy.config.http2.write_size_threshold."""

#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
from ports import get_port
import sys


class TestGrpc():
    """Test proxy.config.http2.write_size_threshold and its associated timeout."""

    def __init__(self, description: str, write_threshold: int, write_timeout: int) -> None:
        """Configure a TestRun for gRPC traffic.

        :param description: The description for the test runs.
        """
        self._description = description
        tr = Test.AddTestRun(self._description)
        dns = self._configure_dns(tr)
        server = self._configure_h2_server(tr, write_timeout)
        ts = self._configure_traffic_server(tr, dns.Variables.Port, server.Variables.port, write_threshold, write_timeout)

        ts.StartBefore(dns)
        ts.StartBefore(server)
        tr.Processes.Default.StartBefore(ts)

        tr.TimeOut = 10

        self._configure_h2_client(tr, ts.Variables.ssl_port, write_timeout)

    def _configure_dns(self, tr: 'TestRun') -> 'Process':
        """Configure a locally running MicroDNS server.

        :param tr: The TestRun with which to associate the MicroDNS server.
        :return: The MicroDNS server process.
        """
        self._dns = tr.MakeDNServer("dns", default=['127.0.0.1'])
        return self._dns

    def _configure_h2_server(self, tr: 'TestRun', write_timeout: int) -> 'Process':
        """Set up the go HTTP/2 server.

        :param tr: The TestRun with which to associate the server.
        :param write_timeout: The expected maximum amount of time frames should be delivered.
        :return: The server process.
        """
        tr.Setup.Copy('trickle_server.py')
        self._server = tr.Processes.Process('server')

        server_pem = os.path.join(Test.Variables.AtsTestToolsDir, "ssl", "server.pem")
        server_key = os.path.join(Test.Variables.AtsTestToolsDir, "ssl", "server.key")
        self._server.Setup.Copy(server_pem)
        self._server.Setup.Copy(server_key)

        port = get_port(self._server, 'port')
        command = (f'{sys.executable} {tr.RunDirectory}/trickle_server.py {port} '
                   f'server.pem server.key {write_timeout}')
        self._server.Command = command
        self._server.ReturnCode = 0
        self._server.Ready = When.PortOpen(port)
        return self._server

    def _configure_traffic_server(
            self, tr: 'TestRun', dns_port: int, server_port: int, write_threshold: int, write_timeout: int) -> 'Process':
        """Configure the traffic server process.

        :param tr: The TestRun with which to associate the traffic server.
        :param dns_port: The MicroDNS server port that traffic server should connect to.
        :param server_port: The server port that traffic server should connect to.
        :param write_threshold: The value to set for proxy.config.http2.write_size_threshold.
        :param write_timeout: The value to set for proxy.config.http2.write_time_threshold.
        :return: The traffic server process.
        """
        self._ts = tr.MakeATSProcess("ts", enable_tls=True, enable_cache=False)

        self._ts.addDefaultSSLFiles()
        self._ts.Disk.ssl_multicert_config.AddLine("dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key")

        self._ts.Disk.remap_config.AddLine(f"map / https://example.com:{server_port}/")

        self._ts.Disk.records_config.update(
            {
                "proxy.config.ssl.server.cert.path": self._ts.Variables.SSLDir,
                "proxy.config.ssl.server.private_key.path": self._ts.Variables.SSLDir,
                'proxy.config.ssl.client.alpn_protocols': 'h2,http/1.1',
                'proxy.config.http.server_session_sharing.pool': 'thread',
                'proxy.config.ssl.client.verify.server.policy': 'PERMISSIVE',
                'proxy.config.dns.nameservers': f"127.0.0.1:{dns_port}",
                'proxy.config.dns.resolv_conf': "NULL",
                'proxy.config.http2.write_size_threshold': write_threshold,
                'proxy.config.http2.write_time_threshold': write_timeout,

                # Only enable debug logging during manual exectution. All the
                # DATA frames get multiple logs and it makes the traffic.out too
                # unwieldy.
                "proxy.config.diags.debug.enabled": 0,
                "proxy.config.diags.debug.tags": "http",
            })
        return self._ts

    def _configure_h2_client(self, tr: 'TestRun', proxy_port: int, write_timeout: int) -> None:
        """Start the HTTP/2 client.

        :param tr: The TestRun with which to associate the client.
        :param proxy_port: The proxy_port to which to connect.
        """
        tr.Setup.Copy('trickle_client.py')
        ca = os.path.join(Test.Variables.AtsTestToolsDir, "ssl", "server.pem")
        key = os.path.join(Test.Variables.AtsTestToolsDir, "ssl", "server.key")

        self._server.Setup.Copy(ca)
        self._server.Setup.Copy(key)
        # The cert is for example.com, so we must use that domain.
        hostname = 'example.com'
        command = (f'{sys.executable} {tr.RunDirectory}/trickle_client.py '
                   f'{hostname} {proxy_port} server.pem {write_timeout}')
        p = tr.Processes.Default
        p.Command = command
        p.ReturnCode = 0


test = TestGrpc("Test proxy.config.http2.write_size_threshold", 0.5, 10)



'''
Test exercising ip_allow configuration of sni.yaml
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''Test sni.yaml ip_allow.'''


class ConnectionType:
    GET = 0
    TUNNEL = 1


class TestSniIpAllow:
    '''Verify ip_allow of sni.yaml.'''
    _dns_counter: int = 0
    _server_counter: int = 0
    _ts_counter: int = 0
    _client_counter: int = 0

    def __init__(self, connect_type: int) -> None:
        """Configure a test run.
        :param connect_type: The type of connection to use.
        """
        tr = Test.AddTestRun("Verify ip_allow of sni.yaml")

        if connect_type == ConnectionType.GET:
            self._replay_file: str = "replay/ip_allow.replay.yaml"
        elif connect_type == ConnectionType.TUNNEL:
            self._replay_file: str = "replay/ip_allow_tunnel.replay.yaml"
        else:
            raise ValueError(f'Invalid connect_type: {connect_type}')

        self._dns = self._configure_dns(tr)
        self._server = self._configure_server(tr)
        self._ts = self._configure_trafficserver(tr, connect_type, self._dns, self._server)
        self._configure_client(tr, self._dns, self._server, self._ts)

    def _configure_dns(self, tr: 'TestRun') -> 'Process':
        """Configure a DNS for the TestRun.
        :param tr: The TestRun to configure with the DNS.
        :return: The DNS Process.
        """
        name = f'dns{TestSniIpAllow._dns_counter}'
        dns = tr.MakeDNServer(name, default='127.0.0.1')
        TestSniIpAllow._dns_counter += 1
        return dns

    def _configure_server(self, tr: 'TestRun') -> 'Process':
        """Configure an Origin Server for the TestRun.
        :param tr: The TestRun to configure with the Origin Server.
        :return: The Origin Server Process.
        """
        name = f'server{TestSniIpAllow._server_counter}'
        server = tr.AddVerifierServerProcess(name, self._replay_file)
        TestSniIpAllow._server_counter += 1
        server.Streams.All += Testers.ContainsExpression('allowed-request', 'The allowed request should be recieved.')
        server.Streams.All += Testers.ExcludesExpression('blocked-request', 'The blocked request should not have been recieved.')
        server.Streams.All += Testers.ExcludesExpression(
            'block.me.com', 'Nothing about the block.me.com sni should have been recieved.')

        return server

    def _configure_trafficserver(self, tr: 'TestRun', connect_type: int, dns: 'Process', server: 'Process') -> 'Process':
        """Configure Traffic Server for the TestRun.
        :param tr: The TestRun to configure with Traffic Server.
        :param connect_type: The type of connection to use.
        :param dns: The DNS Process.
        :param server: The Origin Server Process.
        :return: The Traffic Server Process.
        """
        name = f'ts{TestSniIpAllow._ts_counter}'
        ts = tr.MakeATSProcess(name, enable_tls=True, enable_cache=False)
        TestSniIpAllow._ts_counter += 1
        ts.Disk.sni_yaml.AddLines(
            [
                'sni:',
                '- fqdn: block.me.com',
                '  ip_allow: 192.168.10.1',  # Therefore 127.0.0.1 should be blocked.
            ])
        if connect_type == ConnectionType.TUNNEL:
            ts.Disk.sni_yaml.AddLines([
                f'  tunnel_route: backend.server.com:{server.Variables.https_port}',
            ])
        ts.Disk.sni_yaml.AddLines([
            '- fqdn: allow.me.com',
            '  ip_allow: 127.0.0.1',
        ])
        if connect_type == ConnectionType.TUNNEL:
            ts.Disk.sni_yaml.AddLines([
                f'  tunnel_route: backend.server.com:{server.Variables.https_port}',
            ])
        ts.addDefaultSSLFiles()
        ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')
        ts.Disk.remap_config.AddLine(f'map / http://remapped.backend.server.com:{server.Variables.http_port}/')
        ts.Disk.records_config.update(
            {
                'proxy.config.ssl.server.cert.path': ts.Variables.SSLDir,
                'proxy.config.ssl.server.private_key.path': ts.Variables.SSLDir,
                'proxy.config.ssl.client.verify.server.policy': 'PERMISSIVE',
                'proxy.config.dns.nameservers': f"127.0.0.1:{dns.Variables.Port}",
                'proxy.config.dns.resolv_conf': 'NULL',
                'proxy.config.diags.debug.enabled': 1,
                'proxy.config.diags.debug.tags': 'http|ssl',
            })
        if connect_type == ConnectionType.TUNNEL:
            ts.Disk.records_config.update({
                'proxy.config.http.connect_ports': f"{server.Variables.https_port}",
            })
        return ts

    def _configure_client(self, tr: 'TestRun', dns: 'Process', server: 'Process', ts: 'Process') -> None:
        """Configure the client for the TestRun.
        :param tr: The TestRun to configure with the client.
        :param dns: The DNS Process.
        :param server: The Origin Server Process.
        :param ts: The Traffic Server Process.
        """
        name = f'client{TestSniIpAllow._client_counter}'
        p = tr.AddVerifierClientProcess(
            name, self._replay_file, http_ports=[ts.Variables.port], https_ports=[ts.Variables.ssl_port])
        TestSniIpAllow._client_counter += 1
        ts.StartBefore(server)
        ts.StartBefore(dns)
        p.StartBefore(ts)

        # Because the first connection will be aborted, the client will have a
        # non-zero return code.
        p.ReturnCode = 1

        p.Streams.All += Testers.ContainsExpression('allowed-response', 'The response to the allowed request should be recieved.')
        p.Streams.All += Testers.ExcludesExpression(
            'blocked-response', 'The response to the blocked request should not have been recieved.')


TestSniIpAllow(ConnectionType.GET)
TestSniIpAllow(ConnectionType.TUNNEL)



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test enabling H2 on a per domain basis
'''

Test.SkipUnless(Condition.HasCurlFeature('http2'))

# Define default ATS
ts = Test.MakeATSProcess("ts", enable_tls=True)
server = Test.MakeOriginServer("server")

request_header = {"headers": "GET / HTTP/1.1\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server.addResponse("sessionlog.json", request_header, response_header)

# add ssl materials like key, certificates for the server
ts.addDefaultSSLFiles()

ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.Port))

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

# Set up port 4444 with HTTP1 only, no HTTP/2
ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 0,
        'proxy.config.diags.debug.tags': 'http|ssl',
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.url_remap.pristine_host_hdr': 1,
        'proxy.config.http.server_ports': '{0}:ssl:proto=http {1}'.format(ts.Variables.ssl_port, ts.Variables.port),
        'proxy.config.accept_threads': 0
    })

ts.Disk.sni_yaml.AddLines([
    'sni:',
    '- fqdn: bar.com',
    '  http2: on',
    '- fqdn: "*.foo.com"',
    '  http2: on',
])

tr = Test.AddTestRun("Do-not-Negotiate-h2")
tr.Processes.Default.Command = "curl -v -k --resolve 'foo.com:{0}:127.0.0.1' https://foo.com:{0}".format(ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.Processes.Default.StartBefore(server, ready=When.PortOpen(server.Variables.Port))
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.StillRunningAfter = server
tr.StillRunningAfter = ts
tr.Processes.Default.TimeOut = 5
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("[Uu]sing HTTP/?2", "Curl should negotiate HTTP2")
tr.TimeOut = 5

tr2 = Test.AddTestRun("Do negotiate h2")
tr2.Processes.Default.Command = "curl -v -k --resolve 'bar.com:{0}:127.0.0.1' https://bar.com:{0}".format(ts.Variables.ssl_port)
tr2.ReturnCode = 0
tr2.StillRunningAfter = server
tr2.Processes.Default.TimeOut = 5
tr2.StillRunningAfter = ts
tr2.Processes.Default.Streams.All = Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")
tr2.Processes.Default.Streams.All += Testers.ContainsExpression("[Uu]sing HTTP/?2", "Curl should not negotiate HTTP2")
tr2.TimeOut = 5

tr2 = Test.AddTestRun("Do negotiate h2")
tr2.Processes.Default.Command = "curl -v -k --resolve 'bob.foo.com:{0}:127.0.0.1' https://bob.foo.com:{0}".format(
    ts.Variables.ssl_port)
tr2.ReturnCode = 0
tr2.StillRunningAfter = server
tr2.Processes.Default.TimeOut = 5
tr2.StillRunningAfter = ts
tr2.Processes.Default.Streams.All = Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")
tr2.Processes.Default.Streams.All += Testers.ContainsExpression("[Uu]sing HTTP/?2", "Curl should not negotiate HTTP2")
tr2.TimeOut = 5



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test tls server certificate verification without a pristine host header
'''

# Define default ATS
ts = Test.MakeATSProcess("ts", enable_tls=True)
server_foo = Test.MakeOriginServer(
    "server_foo",
    ssl=True,
    options={
        "--key": "{0}/signed-foo.key".format(Test.RunDirectory),
        "--cert": "{0}/signed-foo.pem".format(Test.RunDirectory)
    })
server = Test.MakeOriginServer("server", ssl=True)
dns = Test.MakeDNServer("dns")

request_foo_header = {"headers": "GET / HTTP/1.1\r\nHost: foo.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
request_bad_foo_header = {"headers": "GET / HTTP/1.1\r\nHost: badfoo.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server_foo.addResponse("sessionlog.json", request_foo_header, response_header)
server_foo.addResponse("sessionlog.json", request_bad_foo_header, response_header)

# add ssl materials like key, certificates for the server
ts.addSSLfile("ssl/signed-foo.pem")
ts.addSSLfile("ssl/signed-foo.key")
ts.addSSLfile("ssl/signed-bar.pem")
ts.addSSLfile("ssl/signed-bar.key")
ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")
ts.addSSLfile("ssl/signer.pem")
ts.addSSLfile("ssl/signer.key")

ts.Disk.remap_config.AddLine(
    'map https://bar.com:{0}/ https://foo.com:{1}'.format(ts.Variables.ssl_port, server_foo.Variables.SSL_Port))
ts.Disk.remap_config.AddLine(
    'map https://foo.com:{0}/ https://bar.com:{1}'.format(ts.Variables.ssl_port, server_foo.Variables.SSL_Port))

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

# Case 1, global config policy=permissive properties=signature
#         override for foo.com policy=enforced properties=all
ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        # set global policy
        'proxy.config.ssl.client.verify.server.policy': 'ENFORCED',
        'proxy.config.ssl.client.verify.server.properties': 'ALL',
        'proxy.config.ssl.client.CA.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.client.CA.cert.filename': 'signer.pem',
        'proxy.config.url_remap.pristine_host_hdr': 0,
        'proxy.config.dns.nameservers': '127.0.0.1:{0}'.format(dns.Variables.Port),
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.dns.resolv_conf': 'NULL'
    })

dns.addRecords(records={"foo.com.": ["127.0.0.1"]})
dns.addRecords(records={"bar.com.": ["127.0.0.1"]})

# bar.com in.  foo.com out.  Should verify
tr = Test.AddTestRun("Enforced-Good-Test")
tr.Setup.Copy("ssl/signed-foo.key")
tr.Setup.Copy("ssl/signed-foo.pem")
tr.Setup.Copy("ssl/signed-bar.key")
tr.Setup.Copy("ssl/signed-bar.pem")
tr.Processes.Default.Command = "curl -v --resolve 'bar.com:{0}:127.0.0.1' -k https://bar.com:{0}".format(ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.Processes.Default.StartBefore(server_foo)
tr.Processes.Default.StartBefore(dns)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.StillRunningAfter = server
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.stdout = Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")

# foo.com in.  bar.com out.  Should not verify
tr2 = Test.AddTestRun("Enforced-bad-test")
tr2.Processes.Default.Command = "curl -v -k --resolve 'foo.com:{0}:127.0.0.1' https://foo.com:{0}".format(ts.Variables.ssl_port)
tr2.ReturnCode = 0
tr2.StillRunningAfter = server
tr2.StillRunningAfter = ts
tr2.Processes.Default.Streams.stdout = Testers.ContainsExpression("Could Not Connect", "Curl attempt should not have succeeded")

# Over riding the built in ERROR check since we expect tr3 to fail
ts.Disk.diags_log.Content = Testers.ExcludesExpression("verification failed", "Make sure the signatures didn't fail")
ts.Disk.diags_log.Content += Testers.ContainsExpression(
    r"WARNING: SNI \(bar.com\) not in certificate", "Make sure bad_bar name checked failed.")



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''

'''
Test.SkipUnless(Condition.PluginExists('cookie_remap.so'))
Test.ContinueOnFail = True
Test.testName = "cookie_remap: Substitute variables (pre-remap URL)"

# Define default ATS
ts = Test.MakeATSProcess("ts")

server = Test.MakeOriginServer("server", ip='127.0.0.10')

request_header = {
    "headers": "GET /photos/search?query=magic HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}

server.addResponse("sessionfile.log", request_header, response_header)

request_header = {
    "headers": "GET /photos/search?query=/theunmatchedpath HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}

server.addResponse("sessionfile.log", request_header, response_header)

request_header = {
    "headers": "GET /photos/search/magic/foobar HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}

server.addResponse("sessionfile.log", request_header, response_header)

request_header = {
    "headers":
        "GET /photos/search/cr_substitutions?query=%28http%3A%2F%2Fwww%2Eexample%2Ecom%2Fmagic HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}

server.addResponse("sessionfile.log", request_header, response_header)

# Setup the remap configuration
config_path = os.path.join(Test.TestDirectory, "configs/psubstituteconfig.txt")
with open(config_path, 'r') as config_file:
    config1 = config_file.read()

ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'cookie_remap.*|http.*|dns.*',
    })

config1 = config1.replace("$PORT", str(server.Variables.Port))

ts.Disk.File(ts.Variables.CONFIGDIR + "/substituteconfig.txt", id="config1")
ts.Disk.config1.WriteOn(config1)

ts.Disk.remap_config.AddLine(
    'map http://www.example.com/magic http://shouldnothit.com/not-used @plugin=cookie_remap.so @pparam=config/substituteconfig.txt')

tr = Test.AddTestRun("Substitute $ppath in the dest query")
tr.Processes.Default.Command = '''
curl \
--proxy 127.0.0.1:{0} \
"http://www.example.com/magic" \
-H"Cookie: fpbeta=abcd" \
-H "Proxy-Connection: keep-alive" \
--verbose \
'''.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.StartBefore(server, ready=When.PortOpen(server.Variables.Port))
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.StillRunningAfter = ts
tr.StillRunningAfter = server

tr = Test.AddTestRun("Substitute $unmatched_ppath in the dest query")
tr.Processes.Default.Command = '''
curl \
--proxy 127.0.0.1:{0} \
"http://www.example.com/magic/theunmatchedpath" \
-H"Cookie: oxalpha=3333" \
-H "Proxy-Connection: keep-alive" \
--verbose \
'''.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.StillRunningAfter = ts
tr.StillRunningAfter = server

tr = Test.AddTestRun("Substitute $cr_req_purl using $cr_urlencode")
tr.Processes.Default.Command = '''
curl \
--proxy 127.0.0.1:{0} \
"http://www.example.com/magic" \
-H"Cookie: acgamma=dfndfdfd" \
-H "Proxy-Connection: keep-alive" \
--verbose \
'''.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.StillRunningAfter = ts
tr.StillRunningAfter = server

tr = Test.AddTestRun("Substitute $ppath as is in outgoing path")
tr.Processes.Default.Command = '''
curl \
--proxy 127.0.0.1:{0} \
"http://www.example.com/magic/foobar" \
-H "Proxy-Connection: keep-alive" \
--verbose \
'''.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.StillRunningAfter = ts
tr.StillRunningAfter = server

server.Streams.All = "gold/psubstitute.gold"



"""
Test child proxy serving stale content when parents are exhausted
"""
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.testName = "proxy_serve_stale"
Test.ContinueOnFail = True


class ProxyServeStaleTest:
    """Verify that stale content is served when the parent is down."""

    single_transaction_replay = "replay/proxy_serve_stale.replay.yaml"
    ts_parent_hostname = "localhost:82"

    def __init__(self):
        """Initialize the test."""
        self._configure_server()
        self._configure_ts()

    def _configure_server(self):
        self.server = Test.MakeVerifierServerProcess("server", self.single_transaction_replay)
        self.nameserver = Test.MakeDNServer("dns", default='127.0.0.1')

    def _configure_ts(self):
        self.ts_child = Test.MakeATSProcess("ts_child")
        # Config child proxy to route to parent proxy
        self.ts_child.Disk.records_config.update(
            {
                'proxy.config.http.push_method_enabled': 1,
                'proxy.config.http.parent_proxy.fail_threshold': 2,
                'proxy.config.http.parent_proxy.total_connect_attempts': 1,
                'proxy.config.http.cache.max_stale_age': 10,
                'proxy.config.http.parent_proxy.self_detect': 0,
                'proxy.config.diags.debug.enabled': 1,
                'proxy.config.diags.debug.tags': 'cache|http|dns|hostdb|parent_proxy',
                'proxy.config.dns.nameservers': f"127.0.0.1:{self.nameserver.Variables.Port}",
            })
        self.ts_child.Disk.parent_config.AddLine(
            f'dest_domain=. parent="{self.ts_parent_hostname}" round_robin=consistent_hash go_direct=false')
        self.ts_child.Disk.remap_config.AddLine(f'map / http://localhost:{self.server.Variables.http_port}')

    def run(self):
        """Run the test cases."""

        tr = Test.AddTestRun()
        tr.AddVerifierClientProcess('client', self.single_transaction_replay, http_ports=[self.ts_child.Variables.port])
        tr.Processes.Default.ReturnCode = 0
        tr.StillRunningAfter = self.ts_child
        tr.Processes.Default.StartBefore(self.server)
        tr.Processes.Default.StartBefore(self.nameserver)
        tr.Processes.Default.StartBefore(self.ts_child)


ProxyServeStaleTest().run()



#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = 'Basic checks on QUIC max_idle_timeout set by ts.quic.no_activity_timeout_in'

Test.SkipUnless(Condition.HasATSFeature('TS_HAS_QUICHE'), Condition.HasCurlFeature('http3'))


class Test_quic_no_activity_timeout:
    """Configure a test for QUIC no_activity_timeout_in."""

    replay_file = "replay/quic_no_activity_timeout.replay.yaml"
    client_counter: int = 0
    ts_counter: int = 0
    server_counter: int = 0

    def __init__(self, name: str, no_activity_timeout_in=1000, gold_file="", replay_keys="", extra_recs=None):
        """Initialize the test.

        :param name: The name of the test.
        :param no_activity_timeout_in: Configuration value for ts.quic.no_activity_timeout_in
        :param gold_file: Gold file to be checked.
        :param replay_keys: Keys to be used by pv
        :param extra_recs: Any additional records to be set, either a yaml string or a dict.
        """
        self.name = name
        self.no_activity_timeout_in = no_activity_timeout_in
        self.gold_file = gold_file
        self.replay_keys = replay_keys
        self.extra_recs = extra_recs

    def _configure_server(self, tr: 'TestRun'):
        """Configure the server.

        :param tr: The TestRun object to associate the server process with.
        """
        server = tr.AddVerifierServerProcess(f"server_{Test_quic_no_activity_timeout.server_counter}", self.replay_file)
        Test_quic_no_activity_timeout.server_counter += 1
        self._server = server

    def _configure_traffic_server(self, tr: 'TestRun'):
        """Configure Traffic Server.

        :param tr: The TestRun object to associate the ts process with.
        """
        ts = tr.MakeATSProcess(f"ts-{Test_quic_no_activity_timeout.ts_counter}", enable_quic=True, enable_tls=True)

        Test_quic_no_activity_timeout.ts_counter += 1
        self._ts = ts
        self._ts.addSSLfile("ssl/cert.crt")
        self._ts.addSSLfile("ssl/private-key.key")
        self._ts.Disk.records_config.update(
            {
                'proxy.config.diags.debug.enabled': 1,
                'proxy.config.diags.debug.tags': 'ssl|net|v_quic|quic|http|socket|inactivity_cop',
                'proxy.config.quic.no_activity_timeout_in': self.no_activity_timeout_in,
                'proxy.config.quic.qlog.file_base': f'log/qlog_{Test_quic_no_activity_timeout.server_counter}',
            })

        if self.extra_recs:
            self._ts.Disk.records_config.update(self.extra_recs)

        self._ts.Disk.remap_config.AddLine(f'map / http://127.0.0.1:{self._server.Variables.http_port}')
        self._ts.Disk.ssl_multicert_config.AddLine(
            f'dest_ip=* ssl_cert_name={self._ts.Variables.SSLDir}/cert.crt ssl_key_name={self._ts.Variables.SSLDir}/private-key.key'
        )

    def run(self, check_for_max_idle_timeout=False):
        """Run the test."""
        tr = Test.AddTestRun(self.name)
        self._configure_server(tr)
        self._configure_traffic_server(tr)

        tr.Processes.Default.StartBefore(self._server)
        tr.Processes.Default.StartBefore(self._ts)

        tr.AddVerifierClientProcess(
            f'client-{Test_quic_no_activity_timeout.client_counter}',
            self.replay_file,
            http3_ports=[self._ts.Variables.ssl_port],
            keys=self.replay_keys)
        Test_quic_no_activity_timeout.client_counter += 1

        if check_for_max_idle_timeout:
            tr.Processes.Default.ReturnCode = 1  # timeout
            self._ts.Disk.traffic_out.Content += Testers.IncludesExpression(
                "QUIC Idle timeout detected", "We should detect the timeout.")
        else:
            tr.Processes.Default.ReturnCode = 0

        if self.gold_file:
            tr.Processes.Default.Streams.all = self.gold_file


# Tests start.

test0 = Test_quic_no_activity_timeout(
    "Test ts.quic.no_activity_timeout_in(quic max_idle_timeout), no delays",
    no_activity_timeout_in=0,  # no timeout `max_idle_timeout`
    replay_keys="nodelays")
test0.run()

test1 = Test_quic_no_activity_timeout(
    "Test ts.quic.no_activity_timeout_in(quic max_idle_timeout) with a 5s delay",
    no_activity_timeout_in=3000,  # 3s `max_idle_timeout`
    replay_keys="delay5s",
    gold_file="gold/quic_no_activity_timeout.gold")
test1.run(check_for_max_idle_timeout=True)

# QUIC Ignores the default_inactivity_timeout config, so the ts.quic.no_activity_timeout_in
# should be honor
test2 = Test_quic_no_activity_timeout(
    "Ignoring default_inactivity_timeout and use the ts.quic.no_activity_timeout_in instead",
    replay_keys="delay5s",
    no_activity_timeout_in=3000,
    extra_recs={'proxy.config.net.default_inactivity_timeout': 1})
test2.run(check_for_max_idle_timeout=True)



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''
Test custom log file format
'''

# this test depends on Linux specific behavior regarding loopback addresses
Test.SkipUnless(Condition.IsPlatform("linux"))

# Define default ATS
ts = Test.MakeATSProcess("ts")

# setup some config file for this server
ts.Disk.remap_config.AddLine('map / http://www.linkedin.com/ @action=deny')

ts.Disk.logging_yaml.AddLines(
    '''
logging:
  formats:
    - name: custom
      format: "%<hii> %<hiih>"
  logs:
    - filename: test_log_field
      format: custom
'''.split("\n"))

# #########################################################################
# at the end of the different test run a custom log file should exist
# Because of this we expect the testruns to pass the real test is if the
# customlog file exists and passes the format check
Test.Disk.File(os.path.join(ts.Variables.LOGDIR, 'test_log_field.log'), exists=True, content='gold/custom.gold')

# first test is a miss for default
tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.0.0.1:{0}" --verbose'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.StartBefore(Test.Processes.ts)

tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.1.1.1:{0}" --verbose'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.2.2.2:{0}" --verbose'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.3.3.3:{0}" --verbose'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.3.0.1:{0}" --verbose'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.43.2.1:{0}" --verbose'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.213.213.132:{0}" --verbose'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.123.32.243:{0}" --verbose'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0

# Wait for log file to appear, then wait one extra second to make sure TS is done writing it.
test_run = Test.AddTestRun()
test_run.Processes.Default.Command = (
    os.path.join(Test.Variables.AtsTestToolsDir, 'condwait') + ' 60 1 -f ' +
    os.path.join(ts.Variables.LOGDIR, 'test_log_field.log'))
test_run.Processes.Default.ReturnCode = 0



'''
Test cached responses and requests with bodies
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test cached responses and requests with bodies
'''

Test.ContinueOnFail = True

# Define default ATS
ts = Test.MakeATSProcess("ts")
server = Test.MakeOriginServer("server")

# **testname is required**
testName = ""
request_header1 = {"headers": "GET / HTTP/1.1\r\nHost: www.example.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header1 = {
    "headers": "HTTP/1.1 200 OK\r\nConnection: close\r\nCache-Control: max-age=300\r\n\r\n",
    "timestamp": "1469733493.993",
    "body": "xxx"
}
request_header2 = {
    "headers": "GET /no_cache_control HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header2 = {
    "headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n",
    "timestamp": "1469733493.993",
    "body": "the flinstones"
}
request_header3 = {
    "headers": "GET /max_age_10sec HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header3 = {
    "headers": "HTTP/1.1 200 OK\r\nConnection: close\r\nCache-Control: max-age=10,public\r\n\r\n",
    "timestamp": "1469733493.993",
    "body": "yabadabadoo"
}
server.addResponse("sessionlog.json", request_header1, response_header1)
server.addResponse("sessionlog.json", request_header2, response_header2)
server.addResponse("sessionlog.json", request_header3, response_header3)

# ATS Configuration
ts.Disk.plugin_config.AddLine('xdebug.so --enable=x-cache,x-cache-key,via')
ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http',
        'proxy.config.http.response_via_str': 3,
        'proxy.config.http.insert_age_in_response': 0,
    })

ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.Port))

# Test 1 - 200 response and cache fill
tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.Command = 'curl -s -D - -v --ipv4 --http1.1 -H "x-debug: x-cache,via" -H "Host: www.example.com" http://localhost:{port}/max_age_10sec'.format(
    port=ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/cache_and_req_body-miss.gold"
tr.StillRunningAfter = ts

# Test 2 - 200 cached response and using netcat
tr = Test.AddTestRun()
tr.Processes.Default.Command = "printf 'GET /max_age_10sec HTTP/1.1\r\n''x-debug: x-cache,x-cache-key,via\r\n''Host: www.example.com\r\n''\r\n'|nc 127.0.0.1 -w 1 {port}".format(
    port=ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/cache_and_req_body-hit.gold"
tr.StillRunningAfter = ts

# Test 3 - response doesn't have cache control directive, so cache-miss every
# time
tr = Test.AddTestRun()
tr.Processes.Default.Command = "printf 'GET /no_cache_control HTTP/1.1\r\n''x-debug: x-cache,x-cache-key,via\r\n''Host: www.example.com\r\n''\r\n'|nc 127.0.0.1 -w 1 {port}".format(
    port=ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/cache_no_cc.gold"
tr.StillRunningAfter = ts

# Test 4 - hit stale cache.
tr = Test.AddTestRun()
tr.Processes.Default.Command = "sleep 15; printf 'GET /max_age_10sec HTTP/1.1\r\n''x-debug: x-cache,x-cache-key,via\r\n''Host: www.example.com\r\n''\r\n'|nc 127.0.0.1 -w 1 {port}".format(
    port=ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/cache_hit_stale.gold"
tr.StillRunningAfter = ts

# Test 5 - only-if-cached. 504 "Not Cached" should be returned if not in cache
tr = Test.AddTestRun()
tr.Processes.Default.Command = "printf 'GET /no_cache_control HTTP/1.1\r\n''Cache-Control: only-if-cached\r\n''x-debug: x-cache,x-cache-key,via\r\n''Host: www.example.com\r\n''Cache-control: max-age=300\r\n''\r\n'|nc 127.0.0.1 -w 1 {port}".format(
    port=ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/cache_no_cache.gold"
tr.StillRunningAfter = ts

#
# Verify correct handling of various max-age directives in both clients and
# responses.
#
ts = Test.MakeATSProcess("ts-for-proxy-verifier")
replay_file = "replay/cache-control-max-age.replay.yaml"
server = Test.MakeVerifierServerProcess("proxy-verifier-server", replay_file)
ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http',
        'proxy.config.http.insert_age_in_response': 0,

        # Disable ignoring max-age in the client request so we can test that
        # behavior too.
        'proxy.config.http.cache.ignore_client_cc_max_age': 0,
    })
ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.http_port))
tr = Test.AddTestRun("Verify correct max-age cache-control behavior.")
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(ts)
tr.AddVerifierClientProcess("proxy-verifier-client", replay_file, http_ports=[ts.Variables.port])

#
# Verify correct handling of various s-maxage directives in responses.
#
ts = Test.MakeATSProcess("ts-s-maxage")
replay_file = "replay/cache-control-s-maxage.replay.yaml"
server = Test.MakeVerifierServerProcess("s-maxage-server", replay_file)
ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http',
        'proxy.config.http.insert_age_in_response': 0,
    })
ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.http_port))
tr = Test.AddTestRun("Verify correct max-age cache-control behavior.")
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(ts)
tr.AddVerifierClientProcess("s-maxage-client", replay_file, http_ports=[ts.Variables.port])

#
# Verify correct interaction between cache-control no-cache and pragma header
#
ts = Test.MakeATSProcess("ts-cache-control-pragma")
ts.Disk.records_config.update({
    'proxy.config.diags.debug.enabled': 1,
    'proxy.config.diags.debug.tags': 'http|cache',
})
tr = Test.AddTestRun("Verify Pragma: no-cache does not conflict with Cache-Control headers")
replay_file = "replay/cache-control-pragma.replay.yaml"
server = tr.AddVerifierServerProcess("pragma-server", replay_file)
tr.AddVerifierClientProcess("pragma-client", replay_file, http_ports=[ts.Variables.port])
ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.http_port))
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(ts)
tr.StillRunningAfter = ts


class RequestCacheControlDefaultTest:
    # Verify the proper handling of cache-control directives in requests in
    # default configuration
    requestCacheControlReplayFile = "replay/request-cache-control-default.replay.yaml"

    def __init__(self):
        self.setupOriginServer()
        self.setupTS()

    def setupOriginServer(self):
        self.server = Test.MakeVerifierServerProcess(
            "request-cache-control-default-verifier-server", self.requestCacheControlReplayFile)

    def setupTS(self):
        self.ts = Test.MakeATSProcess("ts-request-cache-control-default")
        self.ts.Disk.records_config.update({
            "proxy.config.diags.debug.enabled": 1,
            "proxy.config.diags.debug.tags": "http",
        })
        self.ts.Disk.remap_config.AddLine(f"map / http://127.0.0.1:{self.server.Variables.http_port}/",)

    def runTraffic(self):
        tr = Test.AddTestRun("Verify the proper handling of cache-control directives in requests in default configuration")
        tr.AddVerifierClientProcess(
            "request-cache-control-default-client",
            self.requestCacheControlReplayFile,
            http_ports=[self.ts.Variables.port],
            other_args='--thread-limit 1')
        tr.Processes.Default.StartBefore(self.server)
        tr.Processes.Default.StartBefore(self.ts)
        tr.StillRunningAfter = self.server
        tr.StillRunningAfter = self.ts

    def run(self):
        self.runTraffic()


RequestCacheControlDefaultTest().run()


class RequestCacheControlHonorClientTest:
    # Verify the proper handling of cache-control directives in requests when
    # ATS is configured to honor client's request to bypass the cache
    requestCacheControlReplayFile = "replay/request-cache-control-honor-client.replay.yaml"

    def __init__(self):
        self.setupOriginServer()
        self.setupTS()

    def setupOriginServer(self):
        self.server = Test.MakeVerifierServerProcess(
            "request-cache-control-honor-client-verifier-server", self.requestCacheControlReplayFile)

    def setupTS(self):
        self.ts = Test.MakeATSProcess("ts-request-cache-control-honor-client")
        self.ts.Disk.records_config.update(
            {
                "proxy.config.diags.debug.enabled": 1,
                "proxy.config.diags.debug.tags": "http",
                # Configured to honor client requests to bypass the cache
                "proxy.config.http.cache.ignore_client_no_cache": 0
            })
        self.ts.Disk.remap_config.AddLine(f"map / http://127.0.0.1:{self.server.Variables.http_port}/",)

        # Verify logs for the request containing no-cache
        self.ts.Disk.traffic_out.Content += Testers.ContainsExpression(
            "Revalidate document with server", "Verify that ATS honors the no-cache and performs a revalidation.")
        # Verify logs for the request containing no-store
        self.ts.Disk.traffic_out.Content += Testers.ContainsExpression(
            "client does not permit storing, and cache control does not say to ignore client no-cache",
            "Verify that ATS honors the no-store.")

    def runTraffic(self):
        tr = Test.AddTestRun(
            "Verify the proper handling of cache-control directives in requests when ATS is configured to honor client's request to bypass the cache"
        )
        tr.AddVerifierClientProcess(
            "request-cache-control-honor-client-client",
            self.requestCacheControlReplayFile,
            http_ports=[self.ts.Variables.port],
            other_args='--thread-limit 1')
        tr.Processes.Default.StartBefore(self.server)
        tr.Processes.Default.StartBefore(self.ts)
        tr.StillRunningAfter = self.server
        tr.StillRunningAfter = self.ts

    def run(self):
        self.runTraffic()


RequestCacheControlHonorClientTest().run()


class ResponseCacheControlDefaultTest:
    # Verify the proper handling of cache-control directives in responses in
    # default configuration
    responseCacheControlReplayFile = "replay/response-cache-control-default.replay.yaml"

    def __init__(self):
        self.setupOriginServer()
        self.setupTS()

    def setupOriginServer(self):
        self.server = Test.MakeVerifierServerProcess(
            "response-cache-control-default-verifier-server", self.responseCacheControlReplayFile)

    def setupTS(self):
        self.ts = Test.MakeATSProcess("ts-response-cache-control-default")
        self.ts.Disk.records_config.update({
            "proxy.config.diags.debug.enabled": 1,
            "proxy.config.diags.debug.tags": "http",
        })
        self.ts.Disk.remap_config.AddLine(f"map / http://127.0.0.1:{self.server.Variables.http_port}/",)

        # Verify logs for the response containing no-cache
        self.ts.Disk.traffic_out.Content += Testers.ContainsExpression(
            "Revalidate document with server", "Verify that ATS honors the no-cache in response and performs a revalidation.")
        # Verify logs for the response containing no-store
        self.ts.Disk.traffic_out.Content += Testers.ContainsExpression(
            "server does not permit storing and config file does not indicate that server directive should be ignored",
            "Verify that ATS honors the no-store in response and bypasses the cache.")

    def runTraffic(self):
        tr = Test.AddTestRun("Verify the proper handling of cache-control directives in responses in default configuration")
        tr.AddVerifierClientProcess(
            "response-cache-control-client-default",
            self.responseCacheControlReplayFile,
            http_ports=[self.ts.Variables.port],
            other_args='--thread-limit 1')
        tr.Processes.Default.StartBefore(self.server)
        tr.Processes.Default.StartBefore(self.ts)
        tr.StillRunningAfter = self.server
        tr.StillRunningAfter = self.ts

    def run(self):
        self.runTraffic()


ResponseCacheControlDefaultTest().run()


class ResponseCacheControlIgnoredTest:
    # Verify the proper handling of cache-control directives in responses when
    # ATS is configured to ignore server's request to bypass the cache
    responseCacheControlReplayFile = "replay/response-cache-control-ignored.replay.yaml"

    def __init__(self):
        self.setupOriginServer()
        self.setupTS()

    def setupOriginServer(self):
        self.server = Test.MakeVerifierServerProcess(
            "response-cache-control-ignored-verifier-server", self.responseCacheControlReplayFile)

    def setupTS(self):
        self.ts = Test.MakeATSProcess("ts-response-cache-control-ignored")
        self.ts.Disk.records_config.update(
            {
                "proxy.config.diags.debug.enabled": 1,
                "proxy.config.diags.debug.tags": "http",
                "proxy.config.http.cache.ignore_server_no_cache": 1
            })
        self.ts.Disk.remap_config.AddLine(f"map / http://127.0.0.1:{self.server.Variables.http_port}/",)

        # Verify logs for the response containing no-cache or no-store
        self.ts.Disk.traffic_out.Content += Testers.ExcludesExpression(
            "Revalidate document with server",
            "Verify that ATS ignores the no-cache in response and therefore doesn't perform a revalidation.")
        self.ts.Disk.traffic_out.Content += Testers.ExcludesExpression(
            "server does not permit storing and config file does not indicate that server directive should be ignored",
            "Verify that ATS ignores the no-store in response and caches the responses despite its presence.")

    def runTraffic(self):
        tr = Test.AddTestRun(
            "Verify the proper handling of cache-control directives in responses when ATS is configured to ignore server's request to bypass the cache"
        )
        tr.AddVerifierClientProcess(
            "response-cache-control-client-ignored",
            self.responseCacheControlReplayFile,
            http_ports=[self.ts.Variables.port],
            other_args='--thread-limit 1')
        tr.Processes.Default.StartBefore(self.server)
        tr.Processes.Default.StartBefore(self.ts)
        tr.StillRunningAfter = self.server
        tr.StillRunningAfter = self.ts

    def run(self):
        self.runTraffic()


ResponseCacheControlIgnoredTest().run()



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test tls server  certificate verification options. Exercise conf_remap for ca bundle
'''

# Define default ATS
ts = Test.MakeATSProcess("ts")
server1 = Test.MakeOriginServer(
    "server1",
    ssl=True,
    options={
        "--key": "{0}/signed-foo.key".format(Test.RunDirectory),
        "--cert": "{0}/signed-foo.pem".format(Test.RunDirectory)
    })
server2 = Test.MakeOriginServer(
    "server2",
    ssl=True,
    options={
        "--key": "{0}/signed-foo.key".format(Test.RunDirectory),
        "--cert": "{0}/signed2-foo.pem".format(Test.RunDirectory)
    })

request_foo_header = {"headers": "GET / HTTP/1.1\r\nHost: foo.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
request_bad_foo_header = {"headers": "GET / HTTP/1.1\r\nHost: bad_foo.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
request_bar_header = {"headers": "GET / HTTP/1.1\r\nHost: bar.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
request_bad_bar_header = {"headers": "GET / HTTP/1.1\r\nHost: bad_bar.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server1.addResponse("sessionlog.json", request_foo_header, response_header)
server1.addResponse("sessionlog.json", request_bad_foo_header, response_header)
server2.addResponse("sessionlog.json", request_bar_header, response_header)
server2.addResponse("sessionlog.json", request_bad_bar_header, response_header)

# add ssl materials like key, certificates for the server
ts.addSSLfile("ssl/signed-foo.pem")
ts.addSSLfile("ssl/signed2-foo.pem")
ts.addSSLfile("ssl/signed-foo.key")
ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")
ts.addSSLfile("ssl/signer.pem")
ts.addSSLfile("ssl/signer.key")
ts.addSSLfile("ssl/signer2.pem")
ts.addSSLfile("ssl/signer2.key")

ts.Disk.remap_config.AddLine(
    'map /case1 https://127.0.0.1:{0}/ @plugin=conf_remap.so @pparam=proxy.config.ssl.client.CA.cert.filename={1}/{2}'.format(
        server1.Variables.SSL_Port, ts.Variables.SSLDir, "signer.pem"))
ts.Disk.remap_config.AddLine(
    'map /badcase1 https://127.0.0.1:{0}/ @plugin=conf_remap.so @pparam=proxy.config.ssl.client.CA.cert.filename={1}/{2}'.format(
        server1.Variables.SSL_Port, ts.Variables.SSLDir, "signer2.pem"))
ts.Disk.remap_config.AddLine(
    'map /case2 https://127.0.0.1:{0}/ @plugin=conf_remap.so @pparam=proxy.config.ssl.client.CA.cert.filename={1}/{2}'.format(
        server2.Variables.SSL_Port, ts.Variables.SSLDir, "signer2.pem"))
ts.Disk.remap_config.AddLine(
    'map /badcase2 https://127.0.0.1:{0}/ @plugin=conf_remap.so @pparam=proxy.config.ssl.client.CA.cert.filename={1}/{2}'.format(
        server2.Variables.SSL_Port, ts.Variables.SSLDir, "signer.pem"))

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

# Case 1, global config policy=permissive properties=signature
#         override for foo.com policy=enforced properties=all
ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        # set global policy
        'proxy.config.ssl.client.verify.server.policy': 'ENFORCED',
        'proxy.config.ssl.client.verify.server.properties': 'SIGNATURE',
        'proxy.config.ssl.client.CA.cert.path': '/tmp',
        'proxy.config.ssl.client.CA.cert.filename': '{0}/signer.pem'.format(ts.Variables.SSLDir),
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.url_remap.pristine_host_hdr': 1
    })

# Should succeed
tr = Test.AddTestRun("Use correct ca bundle for server 1")
tr.Processes.Default.Command = 'curl -k -H \"host: foo.com\"  http://127.0.0.1:{0}/case1'.format(ts.Variables.port)
tr.ReturnCode = 0
tr.Setup.Copy("ssl/signed-foo.key")
tr.Setup.Copy("ssl/signed-foo.pem")
tr.Setup.Copy("ssl/signed2-foo.pem")
tr.Processes.Default.StartBefore(server1)
tr.Processes.Default.StartBefore(server2)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.StillRunningAfter = server1
tr.StillRunningAfter = ts
# Should succeed.  No message
tr.Processes.Default.Streams.stdout = Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")

tr2 = Test.AddTestRun("Use incorrect ca  bundle for server 1")
tr2.Processes.Default.Command = "curl -k -H \"host: bar.com\"  http://127.0.0.1:{0}/badcase1".format(ts.Variables.port)
tr2.ReturnCode = 0
tr2.StillRunningAfter = server1
tr2.StillRunningAfter = ts
# Should succeed, but will be message in log about name mismatch
tr2.Processes.Default.Streams.stdout = Testers.ContainsExpression("Could Not Connect", "Curl attempt should have succeeded")

tr2 = Test.AddTestRun("Use correct ca bundle for server 2")
tr2.Processes.Default.Command = "curl -k -H \"host: random.com\"  http://127.0.0.1:{0}/case2".format(ts.Variables.port)
tr2.ReturnCode = 0
tr2.StillRunningAfter = server2
tr2.StillRunningAfter = ts
# Should succeed, but will be message in log about signature
tr2.Processes.Default.Streams.stdout = Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")

tr3 = Test.AddTestRun("User incorrect ca bundle for server 2")
tr3.Processes.Default.Command = "curl -k -H \"host: foo.com\"  http://127.0.0.1:{0}/badcase2".format(ts.Variables.port)
tr3.ReturnCode = 0
tr3.StillRunningAfter = server2
tr3.StillRunningAfter = ts
# Should succeed.  No error messages
tr3.Processes.Default.Streams.stdout = Testers.ContainsExpression("Could Not Connect", "Curl attempt should have succeeded")



"""
"""
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = """
Test range requests
"""

Test.ContinueOnFail = True

CACHE_SHORT_MAXAGE = 1


def register(microserver, request_hdr, request_body, response_hdr, response_body):
    request = {
        "headers": "{}\r\n\r\n".format("\r\n".join(line for line in request_hdr.split("\n") if line)),
        "timestamp": "1469733493.993",
        "body": request_body,
    }
    response = {
        "headers": "{}\r\n\r\n".format("\r\n".join(line for line in response_hdr.split("\n") if line)),
        "timestamp": "1469733493.993",
        "body": response_body,
    }
    microserver.addResponse("sessionlog.json", request, response)


def curl_whole(ts, path=""):
    return f"curl -sSv -D - http://127.0.0.1:{ts.Variables.port}/{path}"


def curl_range(ts, path="", ifrange=None, start=1, end=5):
    opt = f"-H 'If-Range: {ifrange}'" if ifrange else ""
    return f"curl -sSv -D - {opt} -H 'Range: bytes={start}-{end}' http://127.0.0.1:{ts.Variables.port}/{path}"


# ----
# Setup Origin Server
# ----
microserver = Test.MakeOriginServer("microserver")

request_hdr = """
GET / HTTP/1.1
Host: 127.0.0.1
"""

response_hdr = """
HTTP/1.1 200 OK
Server: microserver
Connection: close
Cache-Control: max-age=300
Last-Modified: Thu, 10 Feb 2022 00:00:00 GMT
ETag: range
"""

short_cache_request_hdr = """
GET /short HTTP/1.1
Host: 127.0.0.1
"""

short_cache_response_hdr = f"""
HTTP/1.1 200 OK
Server: microserver
Connection: close
Cache-Control: max-age={CACHE_SHORT_MAXAGE}
Last-Modified: Thu, 10 Feb 2022 00:00:00 GMT
ETag: range
"""

response_body = f"{''.join(str(i) for i in range(10))}\n"

register(microserver, request_hdr, "", response_hdr, response_body)
register(microserver, short_cache_request_hdr, "", short_cache_response_hdr, response_body)

# The purpose here is to have a somewhat smarter origin that can respond to If-Modified-Since queries.
# We then can test how the cache server using this origin deals with stale caches.
origin = Test.MakeATSProcess("origin")

origin.Disk.remap_config.AddLine(f"map / http://127.0.0.1:{microserver.Variables.Port}")

origin.Disk.records_config.update(
    {
        "proxy.config.http.cache.http": 1,
        "proxy.config.http.wait_for_cache": 1,
        "proxy.config.http.insert_age_in_response": 0,
        "proxy.config.http.request_via_str": 0,
        "proxy.config.http.response_via_str": 0,
        "proxy.config.diags.debug.enabled": 1,
        "proxy.config.diags.debug.tags": "http",
    })

# Make the origin return 304 Not Modified for stale caches
origin.Disk.cache_config.AddLine("dest_ip=127.0.0.1 pin-in-cache=1d")

# ----
# Setup ATS
# ----
# HACK: Don't use a fixed port because it causes ensuing tests to fail. The problem
# appears to be microDNS not allowing address reuse.
#
# https://docs.python.org/3/library/socketserver.html#socketserver.BaseServer.allow_reuse_address
ts = Test.MakeATSProcess("ts")

ts.Disk.remap_config.AddLine(f"map / http://127.0.0.1:{origin.Variables.port}/")

ts.Disk.records_config.update(
    {
        "proxy.config.http.cache.http": 1,
        "proxy.config.http.cache.range.write": 1,
        "proxy.config.http.response_via_str": 3,
        "proxy.config.http.wait_for_cache": 1,
        "proxy.config.diags.debug.enabled": 1,
        "proxy.config.diags.debug.tags": "http",
    })
# ----
# Test Cases
# ----

# On cache miss
# ---

# ATS should ignore the Range header when given an If-Range header with the incorrect etag
tr = Test.AddTestRun()
tr.Processes.Default.Command = curl_range(ts, ifrange='"should-not-match"')
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.StartBefore(microserver, ready=When.PortOpen(microserver.Variables.Port))
tr.Processes.Default.StartBefore(origin, ready=When.PortOpen(origin.Variables.port))
tr.Processes.Default.StartBefore(Test.Processes.ts, ready=When.PortOpen(ts.Variables.port))
tr.Processes.Default.Streams.stdout = "gold/range-200.gold"

# On cache hit
# ---

# ATS should respond to Range requests with partial content
tr = Test.AddTestRun()
tr.Processes.Default.Command = curl_range(ts)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/range-206.gold"

# ATS should respond to Range requests with partial content when given an If-Range header with
# the correct etag
tr = Test.AddTestRun()
tr.Processes.Default.Command = curl_range(ts, ifrange='"range"')
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/range-206.gold"

# ATS should respond to Range requests with partial content when given an If-Range header
# that matches the Last-Modified header of the cached response
tr = Test.AddTestRun()
tr.Processes.Default.Command = curl_range(ts, ifrange="Thu, 10 Feb 2022 00:00:00 GMT")
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/range-206.gold"

# ATS should respond to Range requests with the full content when given an If-Range header
# that doesn't match the Last-Modified header of the cached response
tr = Test.AddTestRun()
tr.Processes.Default.Command = curl_range(ts, ifrange="Thu, 10 Feb 2022 01:00:00 GMT")
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/range-200.gold"

# ATS should respond to Range requests with a 416 error code when the given Range is invalid
tr = Test.AddTestRun()
tr.Processes.Default.Command = curl_range(ts, start=100, end=105)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/range-416.gold"

# ATS should ignore the Range header when given an If-Range header with the incorrect etag
tr = Test.AddTestRun()
tr.Processes.Default.Command = curl_range(ts, ifrange='"should-not-match"')
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/range-200.gold"

# ATS should ignore the Range header when given an If-Range header with weak etags
tr = Test.AddTestRun()
tr.Processes.Default.Command = curl_range(ts, ifrange='W/"range"')
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/range-200.gold"

# ATS should ignore the Range header when given an If-Range header with a date older than the
# Last-Modified header of the cached response
tr = Test.AddTestRun()
tr.Processes.Default.Command = curl_range(ts, ifrange="Wed, 09 Feb 2022 23:00:00 GMT")
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/range-200.gold"

# Write to the cache by requesting the entire content
tr = Test.AddTestRun()
tr.Processes.Default.Command = curl_whole(ts, path="short")
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/range-200.gold"

# ATS should respond to range requests with partial content for stale caches in response to
# valid If-Range requests if the origin responds with 304 Not Modified.
tr = Test.AddTestRun()
tr.Processes.Default.Command = f"sleep {2 * CACHE_SHORT_MAXAGE}; " + curl_range(ts, path="short", ifrange='"range"')
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/range-206-revalidated.gold"

tr.StillRunningAfter = origin
tr.StillRunningAfter = microserver
tr.StillRunningAfter = ts



'''
For compress gold test, observer for micro-server.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

log = open('compress_userver.log', 'w')


def observe(headers):
    log.write("{}\n".format(headers['X-Ats-Compress-Test']))
    log.flush()


Hooks.register(Hooks.ReadRequestHook, observe)



'''
Test exercising host and SNI mismatch controls
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''
Test exercising host and SNI mismatch controls
'''

ts = Test.MakeATSProcess("ts", enable_tls=True)
cafile = "{0}/signer.pem".format(Test.RunDirectory)
cafile2 = "{0}/signer2.pem".format(Test.RunDirectory)
server = Test.MakeOriginServer("server")

request_header = {"headers": "GET /case1 HTTP/1.1\r\nHost: example.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server.addResponse("sessionlog.json", request_header, response_header)
request_header = {"headers": "GET /  HTTP/1.1\r\nHost: bar.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server.addResponse("sessionlog.json", request_header, response_header)

ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")
ts.addSSLfile("ssl/signer.pem")

ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.url_remap.pristine_host_hdr': 1,
        'proxy.config.ssl.CA.cert.filename': '{0}/signer.pem'.format(ts.Variables.SSLDir),
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.http.host_sni_policy': 2,
        'proxy.config.ssl.TLSv1_3.enabled': 0,
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'ssl',
    })

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

# Just map everything through to origin.  This test is concentrating on the user-agent side
ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}/'.format(server.Variables.Port))

# Scenario 1:  Default no client cert required.  cert required for bar.com.
# Make boblite and bob mixed case to verify that we can match hostnames case
# insensitively.
ts.Disk.sni_yaml.AddLines(
    [
        'sni:',
        '- fqdn: boBliTe',
        '  verify_client: STRICT',
        '  host_sni_policy: PERMISSIVE',
        '- fqdn: bOb',
        '  verify_client: STRICT',
    ])

# case 1
# sni=Bob and host=dave.  Do not provide client cert.  This should match fqdn bOb which has
# verify_client: STRICT and thus should fail.
tr = Test.AddTestRun("Connect to bob without cert")
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.StartBefore(server)
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl -v --tls-max 1.2 -k -H 'host:dave' --resolve 'Bob:{0}:127.0.0.1' https://Bob:{0}/case1".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 35

# case 2
# sni=Bob and host=dave.  Do provide client cert.  This should match fqdn bOb which has
# verify_client: STRICT, but since the cert is good it should succeed.
tr = Test.AddTestRun("Connect to bob with good cert")
tr.Setup.Copy("ssl/signed-foo.pem")
tr.Setup.Copy("ssl/signed-foo.key")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl -v --tls-max 1.2 -k --cert ./signed-foo.pem --key ./signed-foo.key -H 'host:dave' --resolve 'Bob:{0}:127.0.0.1' https://Bob:{0}/case1".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0

# case 3
# sni=dave and host=bob.  Do not provide client cert.  This should not need a cert, but should still
# fail due to sni-host mismatch.
tr = Test.AddTestRun("Connect to dave without cert")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl -v --tls-max 1.2 -k -H 'host:Bob' --resolve 'dave:{0}:127.0.0.1' https://dave:{0}/case1".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression("Access Denied", "Check response")

# case 4
# sni=dave and host=bob.  Do provide client cert.  Again, this should not need a cert, but provide
# one anyway and verify it fails due to sni-host mismatch.
tr = Test.AddTestRun("Connect to dave with cert")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl -v --tls-max 1.2 -k --cert ./signed-foo.pem --key ./signed-foo.key -H 'host:bob' --resolve 'dave:{0}:127.0.0.1' https://dave:{0}/case1".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression("Access Denied", "Check response")

# case 5
# sni=Bob and host=boB.  Do provide client cert.  Should succeed because the hosts match and the cert is provided.
tr = Test.AddTestRun("Connect to bob with cert")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --cert ./signed-foo.pem --key ./signed-foo.key -H 'host:boB' --resolve 'Bob:{0}:127.0.0.1' https://bob:{0}/case1".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("Access Denied", "Check response")

# case 6
# sni=ellen and host=Boblite.  Do not provide client cert.  Should warn due to sni-host mismatch,
# but should note get an Access Denied because boblite has host_sni_policy: PERMISSIVE configured.
tr = Test.AddTestRun("Connect to ellen without cert")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl -v --tls-max 1.2 -k -H 'host:Boblite' --resolve 'ellen:{0}:127.0.0.1' https://ellen:{0}/warnonly".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("Access Denied", "Check response")

# case 7
# sni=ellen and host=Boblite.  Do provide client cert.  This should behave the same as above because
# providing a client cert does not mean SNI and hostname will not be compared.
tr = Test.AddTestRun("Connect to ellen with cert")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl -v --tls-max 1.2 -k --cert ./signed-foo.pem --key ./signed-foo.key -H 'host:Boblite' --resolve 'ellen:{0}:127.0.0.1' https://ellen:{0}/warnonly".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("Access Denied", "Check response")

# case 8
# sni=ellen and host=fran.  Do not provide client cert.  No warning since neither name is mentioned in sni.yaml
tr = Test.AddTestRun("Connect to ellen without cert")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl -v --tls-max 1.2 -k -H 'host:fran' --resolve 'ellen:{0}:127.0.0.1' https://ellen:{0}/warnonly".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("Access Denied", "Check response")

# case 9
# sni=ellen and host=fran.  Do provide client cert.  No warning since neither name is mentioned in sni.yaml
tr = Test.AddTestRun("Connect to ellen with cert")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl -v --tls-max 1.2 -k --cert ./signed-foo.pem --key ./signed-foo.key -H 'host:fran' --resolve 'ellen:{0}:127.0.0.1' https://ellen:{0}/warnonly".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("Access Denied", "Check response")

# Wait for the error.log to appaer.
test_run = Test.AddTestRun()
test_run.Processes.Default.Command = (
    os.path.join(Test.Variables.AtsTestToolsDir, 'condwait') + ' 60 1 -f ' + os.path.join(ts.Variables.LOGDIR, 'error.log'))

ts.Disk.diags_log.Content += Testers.ContainsExpression(
    "WARNING: SNI/hostname mismatch sni=dave host=bob action=terminate", "Should have warning on mismatch")
ts.Disk.diags_log.Content += Testers.ContainsExpression(
    "WARNING: SNI/hostname mismatch sni=ellen host=Boblite action=continue", "Should have warning on mismatch")
ts.Disk.diags_log.Content += Testers.ExcludesExpression(
    "WARNING: SNI/hostname mismatch sni=ellen host=fran", "Should not have warning on mismatch with non-policy host")

test_run.Processes.Default.ReturnCode = 0
ts.Disk.error_log.Content += Testers.ContainsExpression(
    "SNI/hostname mismatch: connecting to .* for host='bob' sni='dave', returning a 403",
    "error.log should contain information about the 403 response.")



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import sys

Test.Summary = '''
Test ATS TLSv1.3 0-RTT support
'''

# Checking only OpenSSL version allows you to run this test with BoringSSL (and it should pass).
Test.SkipUnless(Condition.HasOpenSSLVersion('1.1.1'),)

ts1 = Test.MakeATSProcess('ts1', enable_tls=True)
ts2 = Test.MakeATSProcess('ts2', enable_tls=True)
server = Test.MakeOriginServer('server')

request_header1 = {'headers': 'GET / HTTP/1.1\r\nHost: www.example.com\r\n\r\n', 'timestamp': '1469733493.993', 'body': ''}
response_header1 = {'headers': 'HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n', 'timestamp': '1469733493.993', 'body': 'curl test'}
request_header2 = {'headers': 'GET /early_get HTTP/1.1\r\nHost: www.example.com\r\n\r\n', 'timestamp': '1469733493.993', 'body': ''}
response_header2 = {
    'headers': 'HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n',
    'timestamp': '1469733493.993',
    'body': 'early data accepted'
}
request_header3 = {
    'headers': 'POST /early_post HTTP/1.1\r\nHost: www.example.com\r\nContent-Length: 11\r\n\r\n',
    'timestamp': '1469733493.993',
    'body': 'knock knock'
}
response_header3 = {
    'headers': 'HTTP/1.1 200 OK\r\nServer: uServer\r\nConnection: close\r\nTransfer-Encoding: chunked\r\n\r\n',
    'timestamp': '1415926535.898',
    'body': ''
}
request_header4 = {
    'headers': 'GET /early_multi_1 HTTP/1.1\r\nHost: www.example.com\r\n\r\n',
    'timestamp': '1469733493.993',
    'body': ''
}
response_header4 = {
    'headers': 'HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n',
    'timestamp': '1469733493.993',
    'body': 'early data accepted multi_1'
}
request_header5 = {
    'headers': 'GET /early_multi_2 HTTP/1.1\r\nHost: www.example.com\r\n\r\n',
    'timestamp': '1469733493.993',
    'body': ''
}
response_header5 = {
    'headers': 'HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n',
    'timestamp': '1469733493.993',
    'body': 'early data accepted multi_2'
}
request_header6 = {
    'headers': 'GET /early_multi_3 HTTP/1.1\r\nHost: www.example.com\r\n\r\n',
    'timestamp': '1469733493.993',
    'body': ''
}
response_header6 = {
    'headers': 'HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n',
    'timestamp': '1469733493.993',
    'body': 'early data accepted multi_3'
}
server.addResponse('sessionlog.json', request_header1, response_header1)
server.addResponse('sessionlog.json', request_header2, response_header2)
server.addResponse('sessionlog.json', request_header3, response_header3)
server.addResponse('sessionlog.json', request_header4, response_header4)
server.addResponse('sessionlog.json', request_header5, response_header5)
server.addResponse('sessionlog.json', request_header6, response_header6)

ts1.addSSLfile('ssl/server.pem')
ts1.addSSLfile('ssl/server.key')

ts1.Setup.Copy('test-0rtt-s_client.py')
ts1.Setup.Copy('h2_early_decode.py')
ts1.Setup.Copy('early_h1_get.txt')
ts1.Setup.Copy('early_h1_post.txt')
ts1.Setup.Copy('early_h2_get.txt')
ts1.Setup.Copy('early_h2_post.txt')
ts1.Setup.Copy('early_h2_multi1.txt')
ts1.Setup.Copy('early_h2_multi2.txt')

ts1.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http|ssl_early_data|ssl',
        'proxy.config.exec_thread.autoconfig.enabled': 0,
        'proxy.config.exec_thread.limit': 8,
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts1.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts1.Variables.SSLDir),
        'proxy.config.ssl.session_cache.value': 2,
        'proxy.config.ssl.session_cache.size': 512000,
        'proxy.config.ssl.session_cache.timeout': 7200,
        'proxy.config.ssl.session_cache.num_buckets': 32768,
        'proxy.config.ssl.server.session_ticket.enable': 1,
        'proxy.config.ssl.server.max_early_data': 16384,
        'proxy.config.ssl.server.allow_early_data_params': 0,
        'proxy.config.ssl.server.cipher_suite':
            'ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:DHE-DSS-AES256-GCM-SHA384:DHE-RSA-AES128-GCM-SHA256:DHE-DSS-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA:ECDHE-RSA-AES256-SHA:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-SHA:ECDHE-RSA-AES128-SHA:DHE-RSA-AES256-SHA256:DHE-DSS-AES256-SHA256:DHE-RSA-AES128-SHA256:DHE-DSS-AES128-SHA256:DHE-RSA-AES256-SHA:DHE-DSS-AES256-SHA:DHE-RSA-AES128-SHA:DHE-DSS-AES128-SHA:!aNULL:!eNULL:!EXPORT:!DES:!RC4:!MD5:!PSK:!aECDH:!EDH-DSS-DES-CBC3-SHA:!EDH-RSA-DES-CBC3-SHA:!KRB5-DES-CBC3-SHA'
    })

ts1.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

ts1.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.Port))

ts1.Disk.sni_yaml.AddLines([
    'sni:',
    '- fqdn: example-no.com',
    '  server_max_early_data: 0',
])

ts2.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http|ssl_early_data|ssl',
        'proxy.config.exec_thread.autoconfig.enabled': 0,
        'proxy.config.exec_thread.limit': 8,
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts1.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts1.Variables.SSLDir),
        'proxy.config.ssl.session_cache.value': 2,
        'proxy.config.ssl.session_cache.size': 512000,
        'proxy.config.ssl.session_cache.timeout': 7200,
        'proxy.config.ssl.session_cache.num_buckets': 32768,
        'proxy.config.ssl.server.session_ticket.enable': 1,
        'proxy.config.ssl.server.max_early_data': 0,
        'proxy.config.ssl.server.allow_early_data_params': 0,
        'proxy.config.ssl.server.cipher_suite':
            'ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:DHE-DSS-AES256-GCM-SHA384:DHE-RSA-AES128-GCM-SHA256:DHE-DSS-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA:ECDHE-RSA-AES256-SHA:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-SHA:ECDHE-RSA-AES128-SHA:DHE-RSA-AES256-SHA256:DHE-DSS-AES256-SHA256:DHE-RSA-AES128-SHA256:DHE-DSS-AES128-SHA256:DHE-RSA-AES256-SHA:DHE-DSS-AES256-SHA:DHE-RSA-AES128-SHA:DHE-DSS-AES128-SHA:!aNULL:!eNULL:!EXPORT:!DES:!RC4:!MD5:!PSK:!aECDH:!EDH-DSS-DES-CBC3-SHA:!EDH-RSA-DES-CBC3-SHA:!KRB5-DES-CBC3-SHA'
    })

ts2.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

ts2.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.Port))

ts2.Disk.sni_yaml.AddLines([
    'sni:',
    '- fqdn: example-yes.com',
    '  server_max_early_data: 16384',
])

tr = Test.AddTestRun('Basic Curl Test')
tr.Processes.Default.Command = 'curl -k --resolve example.com:{0}:127.0.0.1 https://example.com:{0}'.format(ts1.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(ts1)
tr.Processes.Default.Streams.All = Testers.ContainsExpression('curl test', 'Making sure the basics still work')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('early data accepted', '')
tr.StillRunningAfter = server
tr.StillRunningAfter += ts1

tr = Test.AddTestRun('TLSv1.3 0-RTT Support (HTTP/1.1 GET)')
tr.Processes.Default.Command = f'{sys.executable} {Test.RunDirectory}/test-0rtt-s_client.py -p {ts1.Variables.ssl_port} -v h1 -t get -r {Test.RunDirectory}'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression('early data accepted', '')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('curl test', '')
tr.StillRunningAfter = server
tr.StillRunningAfter += ts1

tr = Test.AddTestRun('TLSv1.3 0-RTT Support (HTTP/1.1 POST)')
tr.Processes.Default.Command = f'{sys.executable} {Test.RunDirectory}/test-0rtt-s_client.py -p {ts1.Variables.ssl_port} -v h1 -t post -r {Test.RunDirectory}'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression('HTTP/1.1 425 Too Early', '')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('curl test', '')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('early data accepted', '')
tr.StillRunningAfter = server
tr.StillRunningAfter += ts1

tr = Test.AddTestRun('TLSv1.3 0-RTT Support (HTTP/2 GET)')
tr.Processes.Default.Command = f'{sys.executable} {Test.RunDirectory}/test-0rtt-s_client.py -p {ts1.Variables.ssl_port} -v h2 -t get -r {Test.RunDirectory}'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression('early data accepted', '')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('curl test', '')
tr.StillRunningAfter = server
tr.StillRunningAfter += ts1

tr = Test.AddTestRun('TLSv1.3 0-RTT Support (HTTP/2 POST)')
tr.Processes.Default.Command = f'{sys.executable} {Test.RunDirectory}/test-0rtt-s_client.py -p {ts1.Variables.ssl_port} -v h2 -t post -r {Test.RunDirectory}'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression(':status 425', 'Only safe methods are allowed')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('curl test', '')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('early data accepted', '')
tr.StillRunningAfter = server
tr.StillRunningAfter += ts1

tr = Test.AddTestRun('TLSv1.3 0-RTT Support (HTTP/2 Multiplex)')
tr.Processes.Default.Command = f'{sys.executable} {Test.RunDirectory}/test-0rtt-s_client.py -p {ts1.Variables.ssl_port} -v h2 -t multi1 -r {Test.RunDirectory}'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression('early data accepted multi_1', '')
tr.Processes.Default.Streams.All += Testers.ContainsExpression('early data accepted multi_2', '')
tr.Processes.Default.Streams.All += Testers.ContainsExpression('early data accepted multi_3', '')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('curl test', '')
tr.StillRunningAfter = server
tr.StillRunningAfter += ts1

tr = Test.AddTestRun('TLSv1.3 0-RTT Support (HTTP/2 Multiplex with POST)')
tr.Processes.Default.Command = f'{sys.executable} {Test.RunDirectory}/test-0rtt-s_client.py -p {ts1.Variables.ssl_port} -v h2 -t multi2 -r {Test.RunDirectory}'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression('early data accepted multi_1', '')
tr.Processes.Default.Streams.All += Testers.ContainsExpression(':status 425', 'Only safe methods are allowed')
tr.Processes.Default.Streams.All += Testers.ContainsExpression('early data accepted multi_3', '')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('curl test', '')
tr.StillRunningAfter = server
tr.StillRunningAfter += ts1

tr = Test.AddTestRun('TLSv1.3 0-RTT Support (HTTP/1.1 GET) SNI Provided')
tr.Processes.Default.Command = f'{sys.executable} {Test.RunDirectory}/test-0rtt-s_client.py -p {ts1.Variables.ssl_port} -v h1 -t get -r {Test.RunDirectory} -s example.com'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression('early data accepted', '')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('curl test', '')
tr.StillRunningAfter = server
tr.StillRunningAfter += ts1

tr = Test.AddTestRun('TLSv1.3 0-RTT Support (HTTP/1.1 GET) Disabled By SNI Config')
tr.Processes.Default.Command = f'{sys.executable} {Test.RunDirectory}/test-0rtt-s_client.py -p {ts1.Variables.ssl_port} -v h1 -t get -r {Test.RunDirectory} -s example-no.com'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression('early data accepted', '')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('curl test', '')
tr.StillRunningAfter = server

tr = Test.AddTestRun('TLSv1.3 0-RTT Support (HTTP/1.1 GET) Disabled In General')
tr.Processes.Default.Command = f'{sys.executable} {Test.RunDirectory}/test-0rtt-s_client.py -p {ts2.Variables.ssl_port} -v h1 -t get -r {Test.RunDirectory}'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.StartBefore(ts2)
tr.Processes.Default.Streams.All = Testers.ExcludesExpression('early data accepted', '')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('curl test', '')
tr.StillRunningAfter = server
tr.StillRunningAfter += ts2

tr = Test.AddTestRun('TLSv1.3 0-RTT Support (HTTP/1.1 GET) Disabled In General SNI Provided')
tr.Processes.Default.Command = f'{sys.executable} {Test.RunDirectory}/test-0rtt-s_client.py -p {ts2.Variables.ssl_port} -v h1 -t get -r {Test.RunDirectory} -s example.com'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression('early data accepted', '')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('curl test', '')
tr.StillRunningAfter = server
tr.StillRunningAfter += ts2

tr = Test.AddTestRun('TLSv1.3 0-RTT Support (HTTP/1.1 GET) Enabled By SNI Config')
tr.Processes.Default.Command = f'{sys.executable} {Test.RunDirectory}/test-0rtt-s_client.py -p {ts2.Variables.ssl_port} -v h1 -t get -r {Test.RunDirectory} -s example-yes.com'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression('early data accepted', '')
tr.Processes.Default.Streams.All += Testers.ExcludesExpression('curl test', '')



"""
Verify traffic_dump functionality.
"""
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import sys

Test.Summary = '''
Verify traffic_dump functionality.
'''

Test.SkipUnless(Condition.PluginExists('traffic_dump.so'),)

schema_path = os.path.join(Test.Variables.AtsTestToolsDir, 'lib', 'replay_schema.json')
replay_file = "replay/various_sni.yaml"
server = Test.MakeVerifierServerProcess(
    "server-various-sni", replay_file, ssl_cert="ssl/server_combined.pem", ca_cert="ssl/signer.pem")

# Define ATS and configure
ts = Test.MakeATSProcess("ts", enable_tls=True)
replay_dir = os.path.join(ts.RunDirectory, "ts", "log")

ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")
ts.addSSLfile("ssl/signer.pem")

ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'traffic_dump',
        'proxy.config.ssl.server.cert.path': ts.Variables.SSLDir,
        'proxy.config.ssl.server.private_key.path': ts.Variables.SSLDir,
        'proxy.config.url_remap.pristine_host_hdr': 1,
        'proxy.config.ssl.CA.cert.filename': f'{ts.Variables.SSLDir}/signer.pem',
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.http.host_sni_policy': 2,
        'proxy.config.ssl.TLSv1_3.enabled': 0,
        'proxy.config.ssl.client.verify.server.policy': 'PERMISSIVE',
    })

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

ts.Disk.remap_config.AddLine(f'map / https://127.0.0.1:{server.Variables.https_port}')

ts.Disk.sni_yaml.AddLines([
    'sni:',
    '- fqdn: bob.com',
    '  verify_client: NONE',
    '  host_sni_policy: PERMISSIVE',
])

# Configure traffic_dump's SNI filter to only dump connections with SNI bob.com.
sni_filter = "bob.com"
ts.Disk.plugin_config.AddLine(f'traffic_dump.so --logdir {replay_dir} --sample 1 '
                              f'--sni-filter "{sni_filter}"')

# Set up trafficserver expectations.
ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    f"Filtering to only dump connections with SNI: {sni_filter}", "Verify filtering for the expected SNI.")

ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    "Ignore HTTPS session with non-filtered SNI: dave", "Verify that the non-desired SNI session was filtered out.")

ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    "Initialized with sample pool size of 1 bytes and unlimited disk utilization",
    "Verify traffic_dump initialized with the configured disk limit.")

# Set up the json replay file expectations.
replay_file_session_1 = os.path.join(replay_dir, "127", "0000000000000000")
ts.Disk.File(replay_file_session_1, exists=True)

# The second session should be filtered out because it doesn't have the
# expected SNI (note exists is set to False).
replay_file_session_2 = os.path.join(replay_dir, "127", "0000000000000001")
ts.Disk.File(replay_file_session_2, exists=False)

# The third session should also be filtered out because it doesn't have any
# SNI (note exists is set to False).
replay_file_session_2 = os.path.join(replay_dir, "127", "0000000000000002")
ts.Disk.File(replay_file_session_2, exists=False)

# Run the traffic with connections containing various SNI values.
tr = Test.AddTestRun("Test SNI filter with various SNI values in the handshakes.")
# Use the same port across the two servers so that the remap config will work
# across both.
server_port = server.Variables.http_port
tr.AddVerifierClientProcess(
    "client-various-sni",
    replay_file,
    https_ports=[ts.Variables.ssl_port],
    ssl_cert="ssl/server_combined.pem",
    ca_cert="ssl/signer.pem")
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(ts)
tr.StillRunningAfter = server
tr.StillRunningAfter = ts

# Verify the properties of the replay file for the dumped transaction.
tr = Test.AddTestRun("Verify the json content of the first session")
session_1_protocols = "http,tls,tcp,ip"
session_1_tls_features = 'sni:bob.com,proxy-verify-mode:0,proxy-provided-cert:true'
verify_replay = "verify_replay.py"
tr.Setup.CopyAs(verify_replay, Test.RunDirectory)
tr.Processes.Default.Command = \
    (f'{sys.executable} {verify_replay} {schema_path} {replay_file_session_1} '
     f'--client-protocols "{session_1_protocols}" --client-tls-features "{session_1_tls_features}"')
tr.Processes.Default.ReturnCode = 0
